Package: gerda
Title: German Election Database (GERDA)
Version: 0.6.0
Author: Hanno Hilbig [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5849-9172>)
Authors@R: 
    person("Hanno", "Hilbig", , "hhilbig@ucdavis.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-5849-9172"))
Description: Provides tools to download datasets of German elections
    covering local, state, federal, mayoral, European Parliament, and county
    (Kreistag) elections, with federal county-level coverage from 1953 and
    other families extending through 2025. The package supplies turnout, vote
    shares, and derived indicators at the municipal and county level, including
    geographically harmonized datasets that account for changes in municipal
    boundaries over time and incorporate mail-in voting districts. Bundled data
    includes county-level INKAR covariates (1995-2022) and municipality-level
    Zensus 2022 indicators. Data is sourced from
    <https://github.com/awiedem/german_election_data>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5.0)
Imports: dplyr, readr, stats, stringdist, tibble
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
URL: https://github.com/hhilbig/gerda,
        https://github.com/awiedem/german_election_data
BugReports: https://github.com/hhilbig/gerda/issues
VignetteBuilder: knitr
Maintainer: Hanno Hilbig <hhilbig@ucdavis.edu>
NeedsCompilation: no
Packaged: 2026-04-19 23:22:52 UTC; hanno
Repository: CRAN
Date/Publication: 2026-04-20 08:30:08 UTC
Built: R 4.5.2; ; 2026-04-20 23:31:55 UTC; unix

Package: gerda
Title: German Election Database (GERDA)
Version: 0.7.1
Author: Hanno Hilbig [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5849-9172>)
Authors@R: 
    person("Hanno", "Hilbig", , "hhilbig@ucdavis.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-5849-9172"))
Description: Provides tools to download datasets of German elections
    covering local, state, federal, mayoral, European Parliament, and county
    (Kreistag) elections, with federal county-level coverage from 1953 and
    other families extending through 2025. The package supplies turnout, vote
    shares, and derived indicators at the municipal and county level, including
    geographically harmonized datasets that account for changes in municipal
    boundaries over time and incorporate mail-in voting districts. Bundled data
    includes county-level INKAR covariates (1995-2022) and municipality-level
    Zensus 2022 indicators. Data is sourced from
    <https://github.com/awiedem/german_election_data>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5.0)
Imports: dplyr, readr, stats, stringdist, tibble, tools, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
URL: https://github.com/hhilbig/gerda,
        https://github.com/awiedem/german_election_data
BugReports: https://github.com/hhilbig/gerda/issues
VignetteBuilder: knitr
Maintainer: Hanno Hilbig <hhilbig@ucdavis.edu>
NeedsCompilation: no
Packaged: 2026-07-15 17:12:43 UTC; hanno
Repository: CRAN
Date/Publication: 2026-07-15 18:20:10 UTC
Built: R 4.5.2; ; 2026-07-15 22:49:57 UTC; unix

# Generated by roxygen2: do not edit by hand

export(add_gerda_census)
export(add_gerda_covariates)
export(clear_gerda_cache)
export(gerda_cache_dir)
export(gerda_census)
export(gerda_census_codebook)
export(gerda_covariates)
export(gerda_covariates_codebook)
export(gerda_data_list)
export(load_gerda_web)
export(party_crosswalk)
import(dplyr)
import(readr)
import(stringdist)

# GERDA: German Election Data for R

This R package provides data on German elections since 1953, together with helpers for merging socioeconomic covariates. As of v0.7 it exposes 43 datasets covering:

- federal elections at the municipality and county level (1953–2025)
- federal elections at the constituency (Wahlkreis) level (2002–2025), plus a 2021→2025 constituency crosswalk
- state (Landtag) elections, unharmonized and harmonized to 2021, 2023, and 2025 boundaries
- local (municipal) elections, unharmonized and harmonized to 2025 boundaries
- county (Kreistag) elections
- European Parliament elections at the municipal level
- mayoral elections: results, candidates, and person- and municipality-level panels
- AGS and county crosswalks between boundary vintages (1990 through 2025)
- INKAR county covariates (1995–2022) and Zensus 2022 municipality data, both bundled
- a ParlGov party crosswalk

GERDA was compiled by Vincent Heddesheimer, Florian Sichart, Andreas Wiedemann, and Hanno Hilbig. See the [GERDA website](https://www.german-elections.com) and the accompanying publication: [doi.org/10.1038/s41597-025-04811-5](https://doi.org/10.1038/s41597-025-04811-5). The package is under active development; comments and bug reports are welcome at <hhilbig@ucdavis.edu> or via GitHub issues.

## Installation

```r
install.packages("gerda")                  # from CRAN
devtools::install_github("hhilbig/gerda")  # development version
```

To install the vignette along with the development version, pass `build_vignettes = TRUE`:

```r
devtools::install_github("hhilbig/gerda", build_vignettes = TRUE)
```

Then read it with `vignette("gerda")`. The CRAN release ships the vignette by default.

## Main functions

Data access:

- `gerda_data_list()` prints or returns the full list of available datasets with short descriptions.
- `load_gerda_web(file_name, file_format = "rds")` downloads a dataset from the GERDA GitHub repository. `file_format` is `"rds"` (default) or `"csv"`; both return the same tibble, so the choice only affects download size. `file_name` supports fuzzy matching and suggests close alternatives on misspellings.

Bundled data (no download required):

- `gerda_covariates()` and `gerda_covariates_codebook()` for INKAR county covariates and their codebook.
- `gerda_census()` and `gerda_census_codebook()` for Zensus 2022 municipality data and its codebook.

Merging helpers:

- `add_gerda_covariates(election_data)` merges INKAR county covariates into county- or municipality-level election data. On municipality data, all municipalities in the same Kreis receive identical covariate values.
- `add_gerda_census(election_data)` merges Zensus 2022 into county- or municipality-level election data. For county-level merges, municipality values are aggregated: population-weighted means for shares, sums for counts.

Party mapping:

- `party_crosswalk(party_gerda, destination)` maps GERDA party names to a column of ParlGov's `view_party` table (e.g. `"left_right"` for ideology scores).

## Example

```r
library(gerda)
library(dplyr)

federal <- load_gerda_web("federal_muni_harm_25") |>
  add_gerda_covariates() |>
  add_gerda_census()
```

## County covariates (INKAR, 1995–2022)

`add_gerda_covariates()` appends 30 county-level indicators to federal, state, or local election data. Variables cover demographics, GDP and sectoral structure, unemployment (overall, youth, long-term), education, income, healthcare, childcare, housing, transport, and municipal public finances. Coverage is strongest for 1998–2021; newer indicators are available only for recent years. Use `gerda_covariates_codebook()` for per-variable detail including original INKAR codes and missing-data rates.

## Zensus 2022 (municipality-level)

`add_gerda_census()` appends 14 indicators from the German Zensus 2022. Because the census is a single 2022 snapshot, the same values are attached to all election years; analyses that rely on within-unit variation in these variables are not supported.

Indicators cover population and age structure, migration background, household size, and housing (dwellings, vacancy, ownership, rent per square metre, single-family share). Most variables have above 95% municipality coverage. `avg_household_size_census22` is missing for about 12.5% of municipalities because Destatis suppresses small-cell values.

## Deprecations

As of v0.6, `federal_cty_unharm` exposes both the upstream columns (`ags`, `year`) and the canonical GERDA county-level names (`county_code`, `election_year`). The `ags` and `year` aliases will be removed in v0.8. New code should use `county_code` and `election_year`, which match the rest of the county-level datasets and work directly with `add_gerda_covariates()`.

# GERDA Package Test Suite

This directory contains comprehensive tests for the GERDA package, written using the `testthat` framework.

## Test Structure

The test suite is organized into the following files:

### `test-gerda_data_list.R`

Tests for the `gerda_data_list()` function:

- Validates output structure with both `print_table = TRUE/FALSE`
- Tests data integrity and completeness
- Verifies default parameter behavior
- Ensures all expected datasets are present

### `test-load_gerda_web.R`

Tests for the `load_gerda_web()` function:

- Parameter validation (file names, formats)
- Error handling for invalid inputs
- Fuzzy matching functionality for similar dataset names
- Format validation (CSV vs RDS)
- Edge cases and boundary conditions

**Note**: These tests focus on parameter validation and error handling rather than actual data downloading, to avoid network dependencies during testing.

### `test-party_crosswalk.R`

Tests for the `party_crosswalk()` function:

- Basic functionality with known German parties
- Handling of different destination columns (character vs numeric)
- NA value processing
- Parameter validation and error handling
- Edge cases (empty vectors, unknown parties)
- Data type consistency

### `test-integration.R`

Integration tests demonstrating typical package workflows:

- Complete workflow from listing datasets to loading data
- Party crosswalk integration with common German parties
- Error handling across multiple functions
- Data consistency between functions
- Integration with tidyverse/dplyr workflows

## Running the Tests

To run the complete test suite:

```r
# Install and load required packages
library(devtools)

# Run all tests
test()

# Or run specific test files
test_file("tests/testthat/test-gerda_data_list.R")
```

## Test Coverage

The test suite includes **164 individual test cases** covering:

- ✅ All three main package functions
- ✅ Parameter validation and error handling
- ✅ Edge cases and boundary conditions
- ✅ Data integrity and consistency
- ✅ Integration workflows
- ✅ Compatibility with tidyverse tools

## Expected Test Results

When all tests pass, you should see output similar to:

```
[ FAIL 0 | WARN 0 | SKIP 0 | PASS 164 ]
```

## Test Philosophy

These tests are designed to:

1. **Validate functionality** - Ensure all functions work as documented
2. **Prevent regressions** - Catch breaking changes during development  
3. **Document expected behavior** - Serve as executable specifications
4. **Handle edge cases** - Test boundary conditions and error scenarios
5. **Support CI/CD** - Enable automated testing in development workflows

## Dependencies

The tests require the following packages:

- `testthat` (>= 3.0.0) - Testing framework
- `dplyr` - For integration tests with tidyverse workflows

Optional dependencies:

- `devtools` - For running tests during development

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(gerda)

## -----------------------------------------------------------------------------
gerda_data_list()

## ----eval=FALSE---------------------------------------------------------------
# # Load the municipal harmonized dataset
# municipal_harm_data <- load_gerda_web("municipal_harm", verbose = TRUE, file_format = "rds")

## -----------------------------------------------------------------------------
gerda_data_list()

## ----eval=FALSE---------------------------------------------------------------
# federal_cty_harm <- load_gerda_web("federal_cty_harm", verbose = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# library(dplyr)
# 
# # Load election data and add covariates
# merged <- load_gerda_web("federal_cty_harm") %>%
#   add_gerda_covariates()
# 
# # Your data now includes 30 county-level covariates!

## ----eval=FALSE---------------------------------------------------------------
# # Get the codebook
# codebook <- gerda_covariates_codebook()
# print(codebook)
# 
# # Find variables with good coverage
# library(dplyr)
# codebook %>%
#   filter(missing_pct < 10) %>%
#   select(variable, label, category)

## ----eval=FALSE---------------------------------------------------------------
# # Get raw covariate data
# covs <- gerda_covariates()
# 
# # Inspect before merging
# summary(covs$unemployment_rate)
# 
# # Custom merge
# elections <- load_gerda_web("federal_cty_harm")
# merged <- elections %>%
#   left_join(covs, by = c("county_code" = "county_code", "election_year" = "year"))

## ----eval=FALSE---------------------------------------------------------------
# library(gerda)
# 
# # Add census data to municipal-level elections
# muni_merged <- load_gerda_web("federal_muni_harm_21") |>
#   add_gerda_census()
# 
# # Also works with county-level data (aggregated from municipalities)
# county_merged <- load_gerda_web("federal_cty_harm") |>
#   add_gerda_census()

## ----eval=FALSE---------------------------------------------------------------
# # Get the census codebook
# census_cb <- gerda_census_codebook()
# print(census_cb)

## ----eval=FALSE---------------------------------------------------------------
# # Map GERDA party names to left-right positions
# parties <- c("cdu", "spd", "linke_pds", "fdp")
# left_right_scores <- party_crosswalk(parties, "left_right")
# print(left_right_scores)
# 
# # Map to English party names
# english_names <- party_crosswalk(parties, "party_name_english")
# print(english_names)

#' Get Municipality-Level Census 2022 Data
#'
#' @description
#' Returns municipality-level demographic and socioeconomic data from the
#' German Census 2022 (Zensus 2022). This is a cross-sectional snapshot
#' covering all German municipalities.
#'
#' For most users, we recommend using \code{\link{add_gerda_census}} instead,
#' which automatically merges census data with GERDA election data.
#'
#' @return A data frame with approximately 10,800 rows (one per municipality)
#'   and 16 columns containing census indicators. See
#'   \code{\link{gerda_census_codebook}} for variable descriptions.
#'
#' @details
#' The dataset includes:
#' \itemize{
#'   \item Demographics: Population, age structure
#'   \item Migration: Migration background, foreign nationals
#'   \item Households: Average household size
#'   \item Housing: Dwellings, vacancy, ownership, rents, building types
#' }
#'
#' Municipality codes are 8-digit AGS codes. Since the census is a single
#' 2022 snapshot, there is no year dimension.
#'
#' @examples
#' # Get the census data
#' census <- gerda_census()
#' head(census)
#'
#' # Check available municipalities
#' nrow(census)
#'
#' @seealso
#' \itemize{
#'   \item \code{\link{add_gerda_census}} for automatic merging with election data
#'   \item \code{\link{gerda_census_codebook}} for variable descriptions
#' }
#'
#' @export
gerda_census <- function() {
  return(census_2022_internal)
}


#' Get Codebook for Census 2022 Data
#'
#' @description
#' Returns the data dictionary for municipality-level Census 2022 indicators.
#' Provides variable names, labels, units, and data sources.
#'
#' @return A data frame with 16 rows documenting all variables in the census
#'   dataset.
#'
#' @examples
#' # View the codebook
#' codebook <- gerda_census_codebook()
#' print(codebook)
#'
#' @seealso \code{\link{gerda_census}} for the actual census data
#'
#' @export
gerda_census_codebook <- function() {
  return(census_2022_codebook_internal)
}


#' Add Census 2022 Data to GERDA Election Data
#'
#' @description
#' Convenience function to merge Zensus 2022 municipality-level data with
#' GERDA election data. The census provides a cross-sectional snapshot (2022),
#' so the same values are attached to all election years.
#'
#' The function works with both municipality-level and county-level election data:
#' \itemize{
#'   \item \strong{Municipality-level data}: Direct merge using 8-digit AGS codes
#'   \item \strong{County-level data}: Census data is aggregated to the county level
#'         (population-weighted means for shares, sums for counts) before merging
#' }
#'
#' @param election_data A data frame containing GERDA election data. Must contain
#'   either an \code{ags} column (municipality level) or a \code{county_code}
#'   column (county level).
#'
#' @return The input data frame with additional census columns appended. The
#'   number of rows remains unchanged (left join).
#'
#' @details
#' ## Required Columns
#' The input data must contain one of:
#' \itemize{
#'   \item \code{ags}: 8-digit municipal code for municipality-level data
#'   \item \code{county_code}: 5-digit county code for county-level data
#' }
#'
#' ## Merge Behavior
#' Since the census is a 2022 cross-section, census values are the same for all
#' election years. The merge is on geography only (no year join).
#'
#' For county-level data, municipality-level census data is first aggregated:
#' \itemize{
#'   \item Share variables: Population-weighted means
#'   \item Count variables (population_census22, total_dwellings_census22): Sums
#'   \item Other variables (avg_household_size_census22, avg_rent_per_m2_census22): Population-weighted means
#' }
#'
#' @examples
#' \dontrun{
#' library(gerda)
#'
#' # Municipality-level merge
#' muni_data <- load_gerda_web("federal_muni_harm_21") |>
#'   add_gerda_census()
#'
#' # County-level merge (aggregated from municipalities)
#' county_data <- load_gerda_web("federal_cty_harm") |>
#'   add_gerda_census()
#' }
#'
#' @seealso
#' \itemize{
#'   \item \code{\link{gerda_census}} for direct access to the census data
#'   \item \code{\link{gerda_census_codebook}} for variable descriptions
#' }
#'
#' @export
add_gerda_census <- function(election_data) {
  # Avoid NOTE in R CMD check for NSE variables
  ags <- county_code <- county_code_temp <- population_census22 <- NULL

  # Validate input
  if (!is.data.frame(election_data)) {
    stop("election_data must be a data frame")
  }

  has_ags <- "ags" %in% names(election_data)
  has_county_code <- "county_code" %in% names(election_data)

  if (!has_ags && !has_county_code) {
    stop("election_data must contain either 'ags' (for municipal-level data) or 'county_code' (for county-level data)")
  }

  # Get census data
  census <- gerda_census()

  # Remove municipality_name to avoid conflicts
  census_merge <- census %>%
    dplyr::select(-dplyr::any_of("municipality_name"))

  if (has_ags) {
    # Municipality-level: direct merge on AGS
    if (has_county_code) {
      message("Both 'ags' and 'county_code' found. Using 'ags' for municipality-level merge.")
    }

    result <- election_data %>%
      dplyr::left_join(census_merge, by = "ags")

  } else {
    # County-level: aggregate census to county, then merge
    message(
      "Aggregating municipality-level census data to county level.\n",
      "Share variables are population-weighted means; counts are sums."
    )

    # Identify share vs count columns
    share_cols <- grep("^share_|^avg_|vacancy_rate", names(census_merge), value = TRUE)
    count_cols <- c("population_census22", "total_dwellings_census22")

    # Aggregate to county level
    census_county <- census_merge %>%
      dplyr::mutate(county_code_temp = substr(ags, 1, 5)) %>%
      dplyr::group_by(county_code_temp) %>%
      dplyr::summarize(
        dplyr::across(
          dplyr::all_of(share_cols),
          ~ stats::weighted.mean(.x, w = population_census22, na.rm = TRUE),
          .names = "{.col}"
        ),
        dplyr::across(
          dplyr::all_of(count_cols),
          ~ sum(.x, na.rm = TRUE),
          .names = "{.col}"
        ),
        .groups = "drop"
      ) %>%
      dplyr::rename(county_code = county_code_temp)

    result <- election_data %>%
      dplyr::left_join(census_county, by = "county_code")
  }

  return(result)
}

test_that("gerda_census returns a data frame with expected structure", {
  census <- gerda_census()

  expect_s3_class(census, "data.frame")
  expect_gt(nrow(census), 0)
  expect_true("ags" %in% names(census))
  expect_true("population_census22" %in% names(census))
  expect_true("share_migration_bg_census22" %in% names(census))
  expect_true("avg_household_size_census22" %in% names(census))
  expect_true("vacancy_rate_census22" %in% names(census))
})

test_that("gerda_census_codebook returns a data frame with expected structure", {
  codebook <- gerda_census_codebook()

  expect_s3_class(codebook, "data.frame")
  expect_gt(nrow(codebook), 0)
  expect_true(all(c("variable", "label", "unit", "source") %in% names(codebook)))

  # All census data variables should be documented
  census <- gerda_census()
  census_vars <- names(census)
  documented_vars <- codebook$variable
  expect_true(all(census_vars %in% documented_vars))
})

test_that("add_gerda_census validates input", {
  expect_error(add_gerda_census("not a data frame"), "must be a data frame")

  # Missing required columns
  bad_data <- data.frame(x = 1:3)
  expect_error(add_gerda_census(bad_data), "must contain either")
})

test_that("add_gerda_census works with municipality-level data", {
  census <- gerda_census()
  sample_ags <- head(census$ags, 5)

  # Create mock municipal election data
  muni_data <- data.frame(
    ags = sample_ags,
    election_year = 2021,
    votes = c(100, 200, 150, 300, 250),
    stringsAsFactors = FALSE
  )

  result <- add_gerda_census(muni_data)

  # Should keep all original rows
  expect_equal(nrow(result), nrow(muni_data))

  # Should add census columns
  expect_true("population_census22" %in% names(result))
  expect_true("share_migration_bg_census22" %in% names(result))
  expect_true("vacancy_rate_census22" %in% names(result))

  # Original columns should be preserved
  expect_true("votes" %in% names(result))
  expect_true("election_year" %in% names(result))
})

test_that("add_gerda_census works with county-level data", {
  census <- gerda_census()
  # Get some county codes from census AGS
  sample_counties <- unique(substr(census$ags, 1, 5))[1:3]

  county_data <- data.frame(
    county_code = sample_counties,
    election_year = 2021,
    votes = c(1000, 2000, 1500),
    stringsAsFactors = FALSE
  )

  result <- suppressMessages(add_gerda_census(county_data))

  # Should keep all original rows
  expect_equal(nrow(result), nrow(county_data))

  # Should add aggregated census columns (suffixed with _census22)
  census_cols <- grep("_census22$", names(result), value = TRUE)
  expect_gt(length(census_cols), 0)

  # Original columns should be preserved
  expect_true("votes" %in% names(result))
})

test_that("add_gerda_census prefers ags over county_code when both present", {
  census <- gerda_census()
  sample_ags <- head(census$ags, 3)

  both_data <- data.frame(
    ags = sample_ags,
    county_code = substr(sample_ags, 1, 5),
    election_year = 2021,
    stringsAsFactors = FALSE
  )

  # Should use ags and emit a message
  expect_message(add_gerda_census(both_data), "Using 'ags'")
})

#' Get County-Level Covariates from INKAR
#'
#' @description
#' Returns county-level socioeconomic and demographic covariates from INKAR.
#' This function provides flexible access to the raw covariate data for
#' advanced users who want to inspect or manipulate it before merging with
#' county-level election data.
#'
#' For most users, we recommend using \code{\link{add_gerda_covariates}} instead,
#' which automatically performs the merge with correct join keys.
#'
#' \strong{Note}: These covariates are at the county (Kreis) level and should be
#' merged with county-level GERDA data (e.g., \code{federal_cty_harm}).
#'
#' @return A data frame with 11,200 rows and 32 columns containing county-level
#'   covariates for 400 German counties from 1995 to 2022. See
#'   \code{\link{gerda_covariates_codebook}} for variable descriptions.
#'
#' @details
#' The dataset includes 30 socioeconomic and demographic variables:
#' \itemize{
#'   \item Demographics: Age structure, foreign population, gender
#'   \item Economy: GDP, sectoral composition, enterprise structure
#'   \item Labor Market: Unemployment rates (overall, youth, long-term)
#'   \item Education: School completion rates, students, apprentices
#'   \item Income: Purchasing power, low-income households
#'   \item Healthcare: Physician density, hospital beds, GP density
#'   \item Childcare: Coverage rates for under-3 and 3-6 age groups
#'   \item Housing: Building permits, rent levels, living space
#'   \item Transport: Cars per capita
#'   \item Public Finances: Municipal debt, tax revenue
#' }
#'
#' County codes are formatted as 5-digit AGS codes matching GERDA's harmonized
#' county codes (2021 boundaries).
#'
#' @examples
#' # Get the covariates data (bundled, no network call)
#' covs <- gerda_covariates()
#'
#' # Inspect the data
#' head(covs)
#' summary(covs)
#'
#' \donttest{
#' # Manual merge (advanced) — downloads election data from GitHub
#' library(dplyr)
#' elections <- load_gerda_web("federal_cty_harm")
#' merged <- elections %>%
#'   left_join(covs, by = c("county_code" = "county_code", "election_year" = "year"))
#' }
#'
#' @seealso
#' \itemize{
#'   \item \code{\link{add_gerda_covariates}} for automatic merging (recommended)
#'   \item \code{\link{gerda_covariates_codebook}} for variable descriptions
#' }
#'
#' @export
gerda_covariates <- function() {
  return(county_covariates_internal)
}


#' Get Codebook for County-Level Covariates
#'
#' @description
#' Returns the data dictionary for county-level (Kreis) covariates from INKAR.
#' Provides variable names, labels, units, categories, original INKAR codes,
#' and missing data information for all county-level socioeconomic and
#' demographic indicators.
#'
#' @return A data frame with 32 rows documenting all variables in the county
#'   covariates dataset.
#'
#' @examples
#' # View the full codebook
#' codebook <- gerda_covariates_codebook()
#' print(codebook)
#'
#' # Find variables by category
#' library(dplyr)
#' codebook %>%
#'   filter(category == "Demographics")
#'
#' # Find variables with good coverage
#' codebook %>%
#'   filter(missing_pct < 5)
#'
#' @seealso \code{\link{gerda_covariates}} for the actual covariate data
#'
#' @export
gerda_covariates_codebook <- function() {
  return(covariates_codebook_internal)
}


#' Add County-Level Covariates to GERDA Election Data
#'
#' @description
#' Convenience function to merge INKAR county-level (Kreis) covariates with
#' GERDA election data. This is the recommended way to add covariates, as it
#' automatically uses the correct join keys and prevents common merge errors.
#'
#' The function works with both county-level and municipal-level election data:
#' \itemize{
#'   \item \strong{County-level data}: Direct merge using county codes
#'   \item \strong{Municipal-level data}: Automatically extracts county code
#'         from municipal AGS (first 5 digits) and merges
#' }
#'
#' \strong{Important}: Covariates are always at the county level. When merging
#' with municipal data, all municipalities within the same county will receive
#' identical covariate values.
#'
#' The function performs a left join, keeping all rows from the election data
#' and adding covariates where available. This automatically retains only
#' election years.
#'
#' @param election_data A data frame containing GERDA election data. Must
#'   contain a column with county or municipal codes (see Details) and
#'   \code{election_year}.
#'
#' @return The input data frame with additional columns for all 30 county-level
#'   covariates. The number of rows remains unchanged (left join).
#'
#' @details
#' ## Required Columns
#' The input data must contain \code{election_year} and one of:
#' \itemize{
#'   \item \code{county_code}: 5-digit county code (AGS) for county-level data
#'   \item \code{ags}: 8-digit municipal code (AGS) for municipal-level data
#' }
#'
#' The function automatically detects which column is present and performs the
#' appropriate merge. For municipal data, the county code is extracted from the
#' first 5 digits of the AGS.
#'
#' ## Data Level
#' Covariates are at the county (Kreis) level:
#' \itemize{
#'   \item \strong{County-level merge}: One-to-one match, each county gets its covariates
#'   \item \strong{Municipal-level merge}: Many-to-one match, all municipalities in the
#'         same county receive identical covariate values
#' }
#'
#' ## Data Availability
#' Covariates are available from 1995-2022. For GERDA federal elections:
#' \itemize{
#'   \item Elections 1990, 1994: No covariates (before 1995)
#'   \item Elections 1998-2021: Covariates available
#' }
#'
#' ## Missing Data
#' Some covariates have missing values. Use \code{gerda_covariates_codebook()}
#' to check data availability for specific variables.
#'
#' @examples
#' \dontrun{
#' library(gerda)
#' library(dplyr)
#'
#' # Example 1: County-level election data
#' county_data <- load_gerda_web("federal_cty_harm") %>%
#'   add_gerda_covariates()
#'
#' # Check the result
#' names(county_data) # See new covariate columns
#' table(county_data$election_year) # Only election years
#'
#' # Example 2: Municipal-level election data
#' # Note: All municipalities in the same county will get identical covariates
#' muni_data <- load_gerda_web("federal_muni_harm_21") %>%
#'   add_gerda_covariates()
#'
#' # Verify: municipalities in same county have same covariate values.
#' # The county code is the first 5 digits of the 8-digit municipal AGS.
#' muni_data %>%
#'   mutate(county_code = substr(ags, 1, 5)) %>%
#'   group_by(county_code, election_year) %>%
#'   summarize(
#'     n_munis = n(),
#'     unemp_range = max(unemployment_rate) - min(unemployment_rate)
#'   )
#'
#' # Analyze with covariates
#' county_data %>%
#'   filter(election_year == 2021) %>%
#'   filter(!is.na(unemployment_rate)) %>%
#'   summarize(cor_unemployment_afd = cor(unemployment_rate, afd))
#' }
#'
#' @seealso
#' \itemize{
#'   \item \code{\link{gerda_covariates}} for direct access to the covariate data
#'   \item \code{\link{gerda_covariates_codebook}} for variable descriptions
#'   \item \code{\link{load_gerda_web}} for loading GERDA election data
#' }
#'
#' @export
add_gerda_covariates <- function(election_data) {
  # Avoid NOTE in R CMD check for NSE variables
  ags <- county_code_temp <- NULL

  # Validate input
  if (!is.data.frame(election_data)) {
    stop("election_data must be a data frame")
  }

  if (!"election_year" %in% names(election_data)) {
    stop("election_data must contain an 'election_year' column")
  }

  # Detect data level (county or municipal)
  has_county_code <- "county_code" %in% names(election_data)
  has_ags <- "ags" %in% names(election_data)

  if (!has_county_code && !has_ags) {
    stop("election_data must contain either 'county_code' (for county-level data) or 'ags' (for municipal-level data)")
  }

  if (has_county_code && has_ags) {
    message("Both 'county_code' and 'ags' found. Using 'county_code' for merge.")
  }

  # Get covariates
  covs <- gerda_covariates()

  # Handle based on data level
  if (has_county_code) {
    # County-level data: direct merge
    result <- election_data %>%
      dplyr::left_join(
        covs,
        by = c("county_code" = "county_code", "election_year" = "year")
      )
  } else {
    # Municipal-level data: extract county code and merge
    message(
      "Merging county-level covariates to municipal-level data.\n",
      "Note: All municipalities within the same county will have identical covariate values."
    )

    result <- election_data %>%
      dplyr::mutate(county_code_temp = substr(ags, 1, 5)) %>%
      dplyr::left_join(
        covs,
        by = c("county_code_temp" = "county_code", "election_year" = "year")
      ) %>%
      dplyr::select(-county_code_temp)
  }

  return(result)
}

test_that("gerda_covariates returns a data frame with expected structure", {
  covs <- gerda_covariates()

  expect_s3_class(covs, "data.frame")
  expect_gt(nrow(covs), 0)
  expect_true("county_code" %in% names(covs))
  expect_true("year" %in% names(covs))
  expect_true("unemployment_rate" %in% names(covs))
  expect_true("gdp_per_capita" %in% names(covs))
  expect_true("share_foreign" %in% names(covs))
})

test_that("gerda_covariates_codebook returns a data frame with expected structure", {
  codebook <- gerda_covariates_codebook()

  expect_s3_class(codebook, "data.frame")
  expect_gt(nrow(codebook), 0)
  expect_true(all(c("variable", "label") %in% names(codebook)))

  # All covariate variables should be documented
  covs <- gerda_covariates()
  cov_vars <- names(covs)
  documented_vars <- codebook$variable
  expect_true(all(cov_vars %in% documented_vars))
})

test_that("add_gerda_covariates validates input", {
  expect_error(add_gerda_covariates("not a data frame"), "must be a data frame")

  # Missing election_year
  bad_data <- data.frame(county_code = "05111", x = 1)
  expect_error(add_gerda_covariates(bad_data), "election_year")

  # Missing geographic columns
  bad_data2 <- data.frame(election_year = 2017, x = 1)
  expect_error(add_gerda_covariates(bad_data2), "must contain either")
})

test_that("add_gerda_covariates works with county-level data", {
  covs <- gerda_covariates()
  sample_counties <- unique(covs$county_code)[1:5]
  sample_year <- covs$year[1]

  county_data <- data.frame(
    county_code = sample_counties,
    election_year = sample_year,
    votes = c(100, 200, 150, 300, 250),
    stringsAsFactors = FALSE
  )

  result <- add_gerda_covariates(county_data)

  # Should keep all original rows
  expect_equal(nrow(result), nrow(county_data))

  # Should add covariate columns
  expect_true("unemployment_rate" %in% names(result))
  expect_true("gdp_per_capita" %in% names(result))

  # Original columns should be preserved
  expect_true("votes" %in% names(result))
  expect_true("election_year" %in% names(result))
})

test_that("add_gerda_covariates works with municipal-level data", {
  covs <- gerda_covariates()
  sample_county <- unique(covs$county_code)[1]
  sample_year <- covs$year[1]

  # Create mock municipal data with 8-digit AGS derived from a real county code
  muni_data <- data.frame(
    ags = paste0(sample_county, c("001", "002", "003")),
    election_year = sample_year,
    votes = c(100, 200, 150),
    stringsAsFactors = FALSE
  )

  result <- suppressMessages(add_gerda_covariates(muni_data))

  # Should keep all original rows
  expect_equal(nrow(result), nrow(muni_data))

  # Should add covariate columns
  expect_true("unemployment_rate" %in% names(result))

  # All municipalities in same county should get identical covariate values
  unemp_vals <- result$unemployment_rate
  expect_true(length(unique(unemp_vals)) == 1)

  # Temporary county_code_temp column should be removed
  expect_false("county_code_temp" %in% names(result))

  # Original columns should be preserved
  expect_true("votes" %in% names(result))
})

test_that("add_gerda_covariates prefers county_code over ags when both present", {
  covs <- gerda_covariates()
  sample_county <- unique(covs$county_code)[1]
  sample_year <- covs$year[1]

  both_data <- data.frame(
    county_code = sample_county,
    ags = paste0(sample_county, "001"),
    election_year = sample_year,
    stringsAsFactors = FALSE
  )

  # Should use county_code and emit a message
  expect_message(add_gerda_covariates(both_data), "Using 'county_code'")
})

test_that("add_gerda_covariates returns NAs for unmatched years", {
  covs <- gerda_covariates()
  sample_county <- unique(covs$county_code)[1]

  # Use a year outside the covariate range
  county_data <- data.frame(
    county_code = sample_county,
    election_year = 1960,
    stringsAsFactors = FALSE
  )

  result <- add_gerda_covariates(county_data)

  expect_equal(nrow(result), 1)
  expect_true(is.na(result$unemployment_rate))
})

# Schema-normalization tests: the package is expected to rename columns on
# load for known upstream inconsistencies. These tests hit the network.

test_that("federal_cty_unharm exposes county_code/election_year as aliases while keeping ags/year", {
    skip_on_cran()
    data <- tryCatch(
        suppressWarnings(suppressMessages(
            load_gerda_web("federal_cty_unharm", file_format = "rds", verbose = FALSE)
        )),
        error = function(e) NULL
    )
    skip_if(is.null(data), "federal_cty_unharm could not be downloaded (network)")

    expect_s3_class(data, "data.frame")
    # New canonical names, added as aliases.
    expect_true("county_code" %in% names(data))
    expect_true("election_year" %in% names(data))
    # Original upstream names, kept for backwards compatibility (deprecated in v0.7).
    expect_true("ags" %in% names(data))
    expect_true("year" %in% names(data))

    # Alias values should equal the originals.
    expect_identical(data$county_code, data$ags)
    expect_identical(data$election_year, data$year)

    # county_code is a 5-character county AGS.
    sample_codes <- as.character(head(data$county_code, 100))
    code_lengths <- nchar(sample_codes[!is.na(sample_codes)])
    expect_true(all(code_lengths <= 5))
})

test_that("federal_cty_unharm emits a deprecation message on load", {
    skip_on_cran()
    # Capture messages to confirm the one-time deprecation notice fires.
    msgs <- tryCatch(
        capture.output(
            suppressWarnings(
                load_gerda_web("federal_cty_unharm", file_format = "rds", verbose = FALSE)
            ),
            type = "message"
        ),
        error = function(e) NULL
    )
    skip_if(is.null(msgs), "federal_cty_unharm could not be downloaded (network)")
    expect_true(any(grepl("deprecated", msgs, ignore.case = TRUE)))
    expect_true(any(grepl("v0\\.7", msgs)))
})

test_that("federal_cty_unharm can be piped into add_gerda_covariates", {
    skip_on_cran()
    data <- tryCatch(
        suppressWarnings(suppressMessages(
            load_gerda_web("federal_cty_unharm", file_format = "rds", verbose = FALSE)
        )),
        error = function(e) NULL
    )
    skip_if(is.null(data), "federal_cty_unharm could not be downloaded (network)")

    merged <- tryCatch(
        suppressWarnings(suppressMessages(add_gerda_covariates(data))),
        error = function(e) e
    )
    expect_s3_class(merged, "data.frame")
    expect_equal(nrow(merged), nrow(data))
    expect_true("county_code" %in% names(merged))
})

test_that("federal_cty_harm still uses county_code and election_year (no rename)", {
    skip_on_cran()
    data <- tryCatch(
        suppressWarnings(suppressMessages(
            load_gerda_web("federal_cty_harm", file_format = "rds", verbose = FALSE)
        )),
        error = function(e) NULL
    )
    skip_if(is.null(data), "federal_cty_harm could not be downloaded (network)")

    expect_true("county_code" %in% names(data))
    expect_true("election_year" %in% names(data))
})

#' List of GERDA Data
#'
#' This function lists the available GERDA data sets. The purpose of this
#' function is to quickly provide a list of available data sets and their
#' descriptions.
#'
#' In addition to downloadable datasets, the package includes bundled
#' covariate data accessible via dedicated functions:
#' \itemize{
#'   \item \code{\link{gerda_covariates}}: County-level INKAR covariates (1995-2022)
#'   \item \code{\link{gerda_census}}: Municipality-level Census 2022 data
#' }
#'
#' @return A tibble containing the available GERDA data with descriptions. When print_table = TRUE,
#'         the function prints a formatted table to the console and invisibly returns the data tibble.
#'         When print_table = FALSE, the function directly returns the data tibble.
#' @param print_table A logical value indicating whether to print the table in the console (TRUE) or return the data as a tibble (FALSE). Default is TRUE.
#'
#' @examples
#' gerda_data_list()
#'
#' @export
#'
gerda_data_list <- function(print_table = TRUE) {
    if (!isTRUE(print_table) && !isFALSE(print_table)) {
        stop("print_table must be TRUE or FALSE")
    }

    data <- tibble::tribble(
        ~data_name, ~description,
        "municipal_unharm", "Local elections at the municipal level (1990-2020, unharmonized).",
        "municipal_harm", "Local elections at the municipal level (1990-2020, harmonized).",
        "municipal_harm_25", "Local elections at the municipal level, harmonized to 2025 boundaries.",
        "state_unharm", "State elections at the municipal level (2006-2019, unharmonized).",
        "state_harm", "State elections at the municipal level (2006-2019, harmonized).",
        "state_harm_21", "State elections at the municipal level, harmonized to 2021 boundaries.",
        "state_harm_23", "State elections at the municipal level, harmonized to 2023 boundaries.",
        "state_harm_25", "State elections at the municipal level, harmonized to 2025 boundaries.",
        "federal_muni_raw", "Federal elections at the municipal level (1980-2025, raw data).",
        "federal_muni_unharm", "Federal elections at the municipal level (1980-2025, unharmonized).",
        "federal_muni_harm_21", "Federal elections at the municipal level (1990-2025, harmonized to 2021 boundaries).",
        "federal_muni_harm_25", "Federal elections at the municipal level (1990-2025, harmonized to 2025 boundaries).",
        "federal_cty_unharm", "Federal elections at the county level (1953-2021, unharmonized).",
        "federal_cty_harm", "Federal elections at the county level (1990-2021, harmonized).",
        "county_elec_unharm", "County (Kreistag) elections at the municipal level, unharmonized.",
        "county_elec_harm_21", "County (Kreistag) elections, harmonized to 2021 boundaries.",
        "county_elec_harm_21_cty", "County (Kreistag) elections aggregated to county level, harmonized to 2021 boundaries.",
        "county_elec_harm_21_muni", "County (Kreistag) elections at the municipal level, harmonized to 2021 boundaries.",
        "european_muni_unharm", "European Parliament elections at the municipal level, unharmonized.",
        "european_muni_harm", "European Parliament elections at the municipal level, harmonized.",
        "mayoral_unharm", "Mayoral election results at the municipal level, unharmonized.",
        "mayoral_harm", "Mayoral election results at the municipal level, harmonized.",
        "mayoral_candidates", "Mayoral candidates (person-level).",
        "mayor_panel", "Mayor panel (person-level, one row per mayor-term).",
        "mayor_panel_harm", "Mayor panel (person-level, harmonized to current boundaries).",
        "mayor_panel_annual", "Mayor panel at annual frequency (one row per municipality-year).",
        "mayor_panel_annual_harm", "Mayor panel at annual frequency, harmonized to current boundaries.",
        "ags_crosswalks", "Crosswalks for municipalities (1990-2025).",
        "cty_crosswalks", "Crosswalks for counties (1990-2025).",
        "ags_1990_to_2023_crosswalk", "Municipality crosswalk: 1990 boundaries to 2023 boundaries.",
        "ags_1990_to_2025_crosswalk", "Municipality crosswalk: 1990 boundaries to 2025 boundaries.",
        "crosswalk_ags_2021_to_2023", "Municipality crosswalk: AGS 2021 to AGS 2023 (targeted).",
        "crosswalk_ags_2021_2022_to_2023", "Municipality crosswalk: AGS 2021 and 2022 to AGS 2023 (targeted).",
        "crosswalk_ags_2023_to_2025", "Municipality crosswalk: AGS 2023 to AGS 2025 (targeted; RDS only).",
        "crosswalk_ags_2023_24_to_2025", "Municipality crosswalk: AGS 2023 and 2024 to AGS 2025 (targeted; RDS only).",
        "crosswalk_ags_2024_to_2025", "Municipality crosswalk: AGS 2024 to AGS 2025 (targeted; RDS only).",
        "ags_area_pop_emp", "Crosswalk covariates (area, population, employment) for municipalities (1990-2025).",
        "ags_area_pop_emp_2023", "Crosswalk covariates (area, population, employment) for municipalities, harmonized to 2023 boundaries.",
        "cty_area_pop_emp", "Crosswalk covariates (area, population, employment) for counties (1990-2025)."
    )

    if (!print_table) {
        return(data)
    } else {
        # Print one dataset per line as "name  description" so long descriptions
        # are never truncated (the previous kable-based output wrapped or
        # elided, making pairs like federal_muni_harm_21 / _25 indistinguishable
        # by description in narrow terminals).
        name_width <- max(nchar(data$data_name)) + 2L
        for (i in seq_len(nrow(data))) {
            cat(format(data$data_name[i], width = name_width),
                data$description[i], "\n", sep = "")
        }
        invisible(data)
    }
}

test_that("typical workflow: list datasets and understand structure", {
    # This test demonstrates a typical user workflow

    # Step 1: List available datasets
    datasets <- gerda_data_list(print_table = FALSE)

    # Verify we get the expected structure
    expect_s3_class(datasets, "data.frame")
    expect_true(all(c("data_name", "description") %in% colnames(datasets)))
    expect_gt(nrow(datasets), 0)

    # Step 2: Check that dataset names from the list are valid for load_gerda_web.
    # The calls below actually fetch data; skip on CRAN.
    skip_on_cran()
    sample_datasets <- head(datasets$data_name, 3) # Test first 3 datasets

    for (dataset_name in sample_datasets) {
        # These should not produce validation warnings (though may fail at download)
        expect_silent({
            suppressWarnings(load_gerda_web(dataset_name, verbose = FALSE))
        })
    }
})

test_that("party_crosswalk integration with expected GERDA party names", {
    # Test that party_crosswalk works with party names likely to appear in GERDA data

    # Common German party abbreviations/names that should be in the lookup table
    common_parties <- c("cdu", "csu", "spd", "grune", "fdp", "linke_pds")

    # Test mapping to English names
    english_names <- party_crosswalk(common_parties, "party_name_english")
    expect_equal(length(english_names), length(common_parties))
    expect_true(all(!is.na(english_names))) # All should have valid mappings

    # Test mapping to left-right scores
    lr_scores <- party_crosswalk(common_parties, "left_right")
    expect_equal(length(lr_scores), length(common_parties))
    expect_true(all(!is.na(lr_scores))) # All should have valid scores
    expect_true(all(is.numeric(lr_scores)))

    # Test ideological ordering makes sense (basic sanity check)
    cdu_lr <- party_crosswalk("cdu", "left_right")
    spd_lr <- party_crosswalk("spd", "left_right")
    linke_lr <- party_crosswalk("linke_pds", "left_right")

    # CDU should be more right-wing than SPD, SPD more right-wing than Die Linke
    expect_gt(cdu_lr, spd_lr)
    expect_gt(spd_lr, linke_lr)
})

test_that("comprehensive party mapping workflow", {
    # Simulate a realistic workflow where a user maps multiple party attributes

    parties <- c("cdu", "spd", "grune", "fdp", "linke_pds", "csu")

    # Get multiple attributes for the same parties
    party_info <- data.frame(
        party_gerda = parties,
        english_name = party_crosswalk(parties, "party_name_english"),
        short_name = party_crosswalk(parties, "party_name_short"),
        left_right = party_crosswalk(parties, "left_right"),
        family = party_crosswalk(parties, "family_name"),
        stringsAsFactors = FALSE
    )

    # Verify structure
    expect_equal(nrow(party_info), length(parties))
    expect_equal(ncol(party_info), 5)

    # Verify no missing values for major parties
    expect_true(all(!is.na(party_info$english_name)))
    expect_true(all(!is.na(party_info$short_name)))
    expect_true(all(!is.na(party_info$left_right)))
    expect_true(all(!is.na(party_info$family)))

    # Verify data types
    expect_true(is.character(party_info$english_name))
    expect_true(is.character(party_info$short_name))
    expect_true(is.numeric(party_info$left_right))
    expect_true(is.character(party_info$family))
})

test_that("error handling across functions", {
    # Test that functions handle errors gracefully in integrated scenarios

    # Test gerda_data_list with invalid parameters
    expect_error(gerda_data_list(print_table = "invalid"), "print_table must be TRUE or FALSE")

    # Test party_crosswalk with invalid destination from a simulated workflow
    expect_error(
        party_crosswalk(c("cdu", "spd"), "invalid_column"),
        "is not a valid destination"
    )

    # Test load_gerda_web with results from gerda_data_list (downloads on CRAN skip).
    skip_on_cran()
    datasets <- gerda_data_list(print_table = FALSE)
    valid_name <- datasets$data_name[1]
    expect_silent({
        suppressWarnings(load_gerda_web(valid_name))
    })
})

test_that("data consistency across functions", {
    # Test that the dataset names are consistent between functions
    skip_on_cran()

    # Get dataset names from gerda_data_list
    available_datasets <- gerda_data_list(print_table = FALSE)$data_name

    # These names should be valid for load_gerda_web (i.e., no validation warnings)
    for (i in seq_len(min(3, length(available_datasets)))) { # Test first 3
        dataset_name <- available_datasets[i]
        expect_silent({
            # Should pass validation (may fail at download, but that's expected)
            suppressMessages(suppressWarnings(load_gerda_web(dataset_name, verbose = FALSE)))
        })
    }
})

test_that("mixed data types and edge cases integration", {
    # Test realistic scenarios with mixed valid/invalid data

    # Mixed party list with valid, invalid, and NA values
    mixed_parties <- c("cdu", "invalid_party", NA, "spd", "", "grune")

    # Should handle gracefully without errors
    result <- party_crosswalk(mixed_parties, "party_name_english")

    expect_equal(length(result), length(mixed_parties))
    expect_true(is.character(result))

    # Valid parties should have non-NA results
    expect_false(is.na(result[1])) # cdu
    expect_false(is.na(result[4])) # spd
    expect_false(is.na(result[6])) # grune

    # Invalid/missing parties should be NA
    expect_true(is.na(result[2])) # invalid_party
    expect_true(is.na(result[3])) # NA
    expect_true(is.na(result[5])) # empty string
})

test_that("package functions work with tibble/dplyr workflows", {
    # Test integration with common tidyverse workflows
    skip_if_not_installed("dplyr")

    library(dplyr)

    # Create a sample dataset similar to what users might have
    sample_data <- data.frame(
        election_year = c(2017, 2017, 2021, 2021),
        party = c("cdu", "spd", "cdu", "grune"),
        votes = c(1000, 800, 1200, 900),
        stringsAsFactors = FALSE
    )

    # Add party information using party_crosswalk in a dplyr pipeline
    enhanced_data <- sample_data %>%
        mutate(
            party_english = party_crosswalk(party, "party_name_english"),
            left_right_score = party_crosswalk(party, "left_right"),
            party_family = party_crosswalk(party, "family_name")
        )

    # Verify the pipeline worked
    expect_equal(nrow(enhanced_data), nrow(sample_data))
    expect_true("party_english" %in% colnames(enhanced_data))
    expect_true("left_right_score" %in% colnames(enhanced_data))
    expect_true("party_family" %in% colnames(enhanced_data))

    # Verify data integrity
    expect_true(all(!is.na(enhanced_data$party_english)))
    expect_true(all(!is.na(enhanced_data$left_right_score)))
    expect_true(all(!is.na(enhanced_data$party_family)))
})

# Catalog coverage: every exposed dataset name must pass validation and route
# to a load attempt without producing validation warnings. We do not assert
# that the download actually succeeds (network-dependent).

catalog_datasets <- list(
    municipal = c(
        "municipal_unharm",
        "municipal_harm",
        "municipal_harm_25"
    ),
    state = c(
        "state_unharm",
        "state_harm",
        "state_harm_21",
        "state_harm_23",
        "state_harm_25"
    ),
    federal_muni = c(
        "federal_muni_raw",
        "federal_muni_unharm",
        "federal_muni_harm_21",
        "federal_muni_harm_25"
    ),
    federal_cty = c(
        "federal_cty_unharm",
        "federal_cty_harm"
    ),
    county = c(
        "county_elec_unharm",
        "county_elec_harm_21",
        "county_elec_harm_21_cty",
        "county_elec_harm_21_muni"
    ),
    european = c(
        "european_muni_unharm",
        "european_muni_harm"
    ),
    mayoral = c(
        "mayoral_unharm",
        "mayoral_harm",
        "mayoral_candidates",
        "mayor_panel",
        "mayor_panel_harm",
        "mayor_panel_annual",
        "mayor_panel_annual_harm"
    ),
    crosswalks = c(
        "ags_crosswalks",
        "cty_crosswalks",
        "ags_1990_to_2023_crosswalk",
        "ags_1990_to_2025_crosswalk",
        "crosswalk_ags_2021_to_2023",
        "crosswalk_ags_2021_2022_to_2023",
        "crosswalk_ags_2023_to_2025",
        "crosswalk_ags_2023_24_to_2025",
        "crosswalk_ags_2024_to_2025"
    ),
    covariates = c(
        "ags_area_pop_emp",
        "ags_area_pop_emp_2023",
        "cty_area_pop_emp"
    )
)

rds_only <- c(
    "crosswalk_ags_2023_to_2025",
    "crosswalk_ags_2023_24_to_2025",
    "crosswalk_ags_2024_to_2025"
)

for (family in names(catalog_datasets)) {
    local({
        fam  <- family
        dsets <- catalog_datasets[[fam]]
        test_that(paste0("load_gerda_web accepts ", fam, " datasets (n=", length(dsets), ")"), {
            # These tests actually download each dataset from GitHub.
            # Skip on CRAN to avoid flaky network checks and long runtimes.
            skip_on_cran()
            for (ds in dsets) {
                expect_silent({
                    suppressWarnings(suppressMessages(
                        load_gerda_web(ds, verbose = FALSE)
                    ))
                })
            }
        })
    })
}

test_that("all cataloged names appear in gerda_data_list()", {
    listed <- gerda_data_list(print_table = FALSE)$data_name
    cataloged <- unlist(catalog_datasets, use.names = FALSE)
    expect_setequal(cataloged, listed)
})

test_that("catalog total matches the data_list row count", {
    total <- length(unlist(catalog_datasets, use.names = FALSE))
    listed_n <- nrow(gerda_data_list(print_table = FALSE))
    expect_equal(total, listed_n)
})

test_that("RDS-only datasets can be requested with file_format='rds'", {
    skip_on_cran()
    for (ds in rds_only) {
        expect_silent({
            suppressWarnings(suppressMessages(
                load_gerda_web(ds, file_format = "rds", verbose = FALSE)
            ))
        })
    }
})

test_that("xz-compressed RDS loads (regression: ags_1990_to_2025_crosswalk)", {
    # Some upstream RDS files are xz-compressed, which readr::read_rds cannot
    # stream from a URL. load_gerda_web must download to a tempfile so base
    # readRDS() can auto-detect the compression.
    skip_on_cran()
    data <- tryCatch(
        suppressWarnings(suppressMessages(
            load_gerda_web("ags_1990_to_2025_crosswalk", file_format = "rds", verbose = FALSE)
        )),
        error = function(e) NULL
    )
    skip_if(is.null(data), "ags_1990_to_2025_crosswalk could not be downloaded (network)")
    expect_s3_class(data, "data.frame")
    expect_gt(nrow(data), 1e5)  # upstream file is ~451,772 rows
})

test_that("gerda_data_list returns correct structure with print_table = TRUE", {
    # Capture output to test both printed table and returned data
    output <- capture.output(result <- gerda_data_list(print_table = TRUE))

    # Test that result is a tibble
    expect_s3_class(result, "tbl_df")
    expect_s3_class(result, "data.frame")

    # Test that result has expected columns
    expect_true(all(c("data_name", "description") %in% colnames(result)))

    # Test that result has expected number of rows (39 datasets as of current version)
    expect_equal(nrow(result), 39)

    # Test that output was captured (meaning something was printed)
    expect_gt(length(output), 0)

    # Test that specific datasets are present
    expected_datasets <- c(
        "municipal_unharm", "municipal_harm", "state_unharm",
        "state_harm", "federal_muni_raw", "federal_muni_unharm",
        "federal_muni_harm_21", "federal_muni_harm_25",
        "federal_cty_unharm", "federal_cty_harm",
        "ags_crosswalks", "cty_crosswalks", "ags_area_pop_emp",
        "cty_area_pop_emp"
    )
    expect_true(all(expected_datasets %in% result$data_name))
})

test_that("gerda_data_list returns correct structure with print_table = FALSE", {
    result <- gerda_data_list(print_table = FALSE)

    # Test that result is a tibble
    expect_s3_class(result, "tbl_df")
    expect_s3_class(result, "data.frame")

    # Test that result has expected columns
    expect_true(all(c("data_name", "description") %in% colnames(result)))

    # Test that result has expected number of rows
    expect_equal(nrow(result), 39)

    # Test that no output is printed when print_table = FALSE
    output <- capture.output(result2 <- gerda_data_list(print_table = FALSE))
    expect_equal(length(output), 0)
})

test_that("gerda_data_list data integrity", {
    result <- gerda_data_list(print_table = FALSE)

    # Test that all data_name entries are non-empty strings
    expect_true(all(nchar(result$data_name) > 0))
    expect_true(all(is.character(result$data_name)))

    # Test that all description entries are non-empty strings
    expect_true(all(nchar(result$description) > 0))
    expect_true(all(is.character(result$description)))

    # Test that data_name entries are unique
    expect_equal(length(result$data_name), length(unique(result$data_name)))

    # Test specific descriptions contain expected content
    expect_true(any(grepl("Local elections", result$description)))
    expect_true(any(grepl("State elections", result$description)))
    expect_true(any(grepl("Federal elections", result$description)))
    expect_true(any(grepl("harmonized", result$description)))
    expect_true(any(grepl("unharmonized", result$description)))
})

test_that("gerda_data_list default parameter behavior", {
    # Test that default behavior (no parameters) works like print_table = TRUE
    output1 <- capture.output(result1 <- gerda_data_list())
    output2 <- capture.output(result2 <- gerda_data_list(print_table = TRUE))

    expect_equal(result1, result2)
    expect_gt(length(output1), 0)
    expect_gt(length(output2), 0)
})

#' Map GERDA Party Names to ParlGov Attributes
#'
#' @description
#' Creates a crosswalk between GERDA party names and ParlGov's `view_party` attributes.
#' If a party name is not found, the corresponding output element is `NA`. This function
#' expects GERDA party names (lowercase, underscores); other naming schemes will mostly
#' return `NA`.
#'
#' @param party_gerda A character vector containing the GERDA party names to be converted.
#' @param destination A single string naming the target column. Available destinations:
#'   \itemize{
#'     \item \strong{Names}: `party_name`, `party_name_ascii`, `party_name_short`, `party_name_english`
#'     \item \strong{Party family}: `family_name`, `family_name_short`
#'     \item \strong{Ideology scales (ParlGov)}: `left_right`, `state_market`, `liberty_authority`, `eu_anti_pro`
#'     \item \strong{External ideology scores}: `cmp`, `euprofiler`, `ees`, `castles_mair`, `huber_inglehart`, `ray`, `benoit_laver`, `chess`
#'     \item \strong{Identifiers}: `country_id`, `party_id`, `family_id`
#'   }
#' @return A vector of the same length as `party_gerda` with the mapped values.
#' @export
#' @examples
#' party_crosswalk(c("cdu", "spd", "linke_pds", NA), "left_right")
#' party_crosswalk(c("cdu", "afd"), "family_name_short")
#'
party_crosswalk <- function(party_gerda, destination) {
    lt <- lookup_table
    available <- setdiff(colnames(lt), "party_gerda")

    # Length of destination must be 1
    if (length(destination) != 1) {
        stop("destination must be a single character string. Available destinations: ",
             paste(available, collapse = ", "))
    }

    # Check if destination is a column name of lt
    if (!destination %in% colnames(lt)) {
        stop("'", destination, "' is not a valid destination. Available destinations: ",
             paste(available, collapse = ", "))
    }

    # Check if party_gerda is a character vector
    if (!is.character(party_gerda)) {
        stop("party_gerda must be a character vector")
    }

    # Determine the type of NA to use based on the destination column
    na_value <- if (is.numeric(lt[[destination]])) NA_real_ else NA_character_

    # Match party_gerda values with lookup table and return corresponding destination values
    lookup_result <- lt[[destination]][match(party_gerda, lt$party_gerda)]

    # Replace any NA results with the appropriate NA type
    lookup_result[is.na(lookup_result)] <- na_value

    return(lookup_result)
}

test_that("party_crosswalk basic functionality works", {
    # Test basic party mapping with known parties
    result <- party_crosswalk(c("cdu", "spd", "grune"), "party_name_english")

    expect_equal(length(result), 3)
    expect_true(is.character(result))
    expect_true("Christian Democratic Union" %in% result)
    expect_true("Social Democratic Party of Germany" %in% result)
    expect_true("Alliance 90 / Greens" %in% result)
})

test_that("party_crosswalk handles numeric destinations", {
    # Test with numeric destination (left_right scores)
    result <- party_crosswalk(c("cdu", "spd"), "left_right")

    expect_equal(length(result), 2)
    expect_true(is.numeric(result))
    expect_true(all(!is.na(result)))

    # CDU should be more right-wing than SPD
    cdu_score <- party_crosswalk("cdu", "left_right")
    spd_score <- party_crosswalk("spd", "left_right")
    expect_gt(cdu_score, spd_score)
})

test_that("party_crosswalk handles NA values correctly", {
    # Test that NA input produces appropriate NA output
    result_char <- party_crosswalk(c("cdu", NA, "spd"), "party_name_english")
    expect_equal(length(result_char), 3)
    expect_true(is.na(result_char[2]))
    expect_true(is.character(result_char[2]))

    result_num <- party_crosswalk(c("cdu", NA, "spd"), "left_right")
    expect_equal(length(result_num), 3)
    expect_true(is.na(result_num[2]))
    expect_true(is.numeric(result_num[2]))
})

test_that("party_crosswalk handles unknown parties", {
    # Test that unknown parties return NA
    result <- party_crosswalk(c("cdu", "unknown_party", "spd"), "party_name_english")

    expect_equal(length(result), 3)
    expect_false(is.na(result[1])) # cdu should be found
    expect_true(is.na(result[2])) # unknown_party should be NA
    expect_false(is.na(result[3])) # spd should be found
})

test_that("party_crosswalk validates destination parameter", {
    # Unknown destination: error names it and lists available destinations.
    expect_error(
        party_crosswalk(c("cdu", "spd"), "nonexistent_column"),
        "is not a valid destination"
    )
    expect_error(
        party_crosswalk(c("cdu", "spd"), "nonexistent_column"),
        "Available destinations"
    )
    expect_error(
        party_crosswalk(c("cdu", "spd"), "nonexistent_column"),
        "left_right"
    )

    # Multiple destination values: single-string requirement is still enforced.
    expect_error(
        party_crosswalk(c("cdu", "spd"), c("left_right", "party_name")),
        "single character string"
    )
    expect_error(
        party_crosswalk(c("cdu", "spd"), c("left_right", "party_name")),
        "Available destinations"
    )
})

test_that("party_crosswalk validates party_gerda parameter", {
    # Test non-character input
    expect_error(
        party_crosswalk(c(1, 2, 3), "left_right"),
        "party_gerda must be a character vector"
    )

    expect_error(
        party_crosswalk(123, "left_right"),
        "party_gerda must be a character vector"
    )

    expect_error(
        party_crosswalk(TRUE, "left_right"),
        "party_gerda must be a character vector"
    )
})

test_that("party_crosswalk preserves vector length", {
    # Test that output length always matches input length
    inputs <- list(
        character(0), # empty vector
        "cdu", # single element
        c("cdu", "spd"), # two elements
        c("cdu", "spd", "grune", "fdp", "linke_pds"), # multiple elements
        c("cdu", NA, "unknown", "spd") # mixed valid/invalid/NA
    )

    for (input in inputs) {
        result <- party_crosswalk(input, "party_name_english")
        expect_equal(length(result), length(input))
    }
})

test_that("party_crosswalk works with different destination columns", {
    # Test various destination columns that should exist
    test_party <- "cdu"

    destinations_char <- c(
        "party_name_ascii", "party_name_short",
        "party_name_english", "family_name_short", "family_name"
    )
    destinations_num <- c(
        "left_right", "state_market", "liberty_authority",
        "eu_anti_pro", "country_id", "party_id"
    )

    # Test character destinations
    for (dest in destinations_char) {
        result <- party_crosswalk(test_party, dest)
        expect_true(is.character(result))
        expect_equal(length(result), 1)
    }

    # Test numeric destinations
    for (dest in destinations_num) {
        result <- party_crosswalk(test_party, dest)
        expect_true(is.numeric(result))
        expect_equal(length(result), 1)
    }
})

test_that("party_crosswalk edge cases", {
    # Test empty character vector
    result <- party_crosswalk(character(0), "party_name_english")
    expect_equal(length(result), 0)
    expect_true(is.character(result))

    # Test vector of all NAs (convert to character to avoid type issues)
    result <- party_crosswalk(as.character(c(NA, NA, NA)), "left_right")
    expect_equal(length(result), 3)
    expect_true(all(is.na(result)))
    expect_true(is.numeric(result))

    # Test vector of all unknown parties
    result <- party_crosswalk(c("unknown1", "unknown2"), "party_name_english")
    expect_equal(length(result), 2)
    expect_true(all(is.na(result)))
    expect_true(is.character(result))
})

test_that("party_crosswalk specific party mappings", {
    # Test specific known party mappings to ensure data integrity
    expect_equal(party_crosswalk("cdu", "party_name_short"), "CDU")
    expect_equal(party_crosswalk("spd", "party_name_short"), "SPD")
    expect_equal(party_crosswalk("csu", "party_name_short"), "CSU")

    # Test that major parties have left-right scores
    major_parties <- c("cdu", "csu", "spd", "grune", "fdp", "linke_pds")
    lr_scores <- party_crosswalk(major_parties, "left_right")
    expect_true(all(!is.na(lr_scores)))
    expect_true(all(lr_scores >= 0 & lr_scores <= 10)) # Typical left-right scale
})

#' Load GERDA Data
#'
#' This function loads GERDA data from a web source.
#'
#' @param file_name A character string specifying the name of the file to load. For a list of available data, see \code{\link{gerda_data_list}}.
#' @param verbose A logical value indicating whether to print additional messages to the console. Default is FALSE.
#' @param file_format A character string specifying the format of the file. Must be either "csv" or "rds". Default is "rds".
#' @param on_error How to handle errors (unknown dataset name, failed download, corrupt file, invalid `file_format`). Either `"warn"` (default) to emit a warning and return `NULL`, or `"stop"` to raise an error. Use `"stop"` inside scripts or pipelines where silent `NULL` returns would produce confusing downstream failures. The global default can also be overridden with `options(gerda.on_error = "stop")`.
#'
#' @section Vote-share columns:
#' Election datasets expose one column per party (e.g. `cdu`, `spd`, `gruene`, `afd`).
#' These columns hold the party's share of valid votes and are expressed as
#' fractions of 1. They do **not** sum to 1 across the named major parties:
#' the remainder is held by smaller parties with their own columns and, at
#' the tail, an `other` category. For example, in `federal_cty_harm` for 2021,
#' `cdu + csu + spd + gruene + fdp + linke_pds + afd` is typically around
#' 0.91 and ranges roughly 0.78 to 0.97 across counties. To reconstruct a
#' full 1.0 share, include every party column or use `other` together with
#' turnout and invalid-vote columns.
#'
#' @return A tibble containing the loaded data, or NULL if the data could not be loaded.
#'
#' @examples
#' \donttest{
#' # Load harmonized municipal elections data
#' data_municipal_harm <- load_gerda_web("municipal_harm", verbose = TRUE, file_format = "rds")
#'
#' # Load federal election data harmonized to 2025 boundaries (includes 2025 election)
#' data_federal_2025 <- load_gerda_web("federal_muni_harm_25", verbose = TRUE, file_format = "rds")
#' }
#'
#' @import dplyr
#' @import stringdist
#' @import readr
#' @export

load_gerda_web <- function(file_name,
                           verbose = FALSE,
                           file_format = "rds",
                           on_error = getOption("gerda.on_error", "warn")) {
    if (!is.character(file_name) || length(file_name) != 1 || nchar(file_name) == 0) {
        stop("file_name must be a single non-empty character string")
    }

    if (!is.character(on_error) || length(on_error) != 1 || !on_error %in% c("warn", "stop")) {
        stop("on_error must be either \"warn\" or \"stop\"")
    }

    # Internal helper: every failure path goes through this so the on_error
    # contract is consistent (warn + NULL, or stop).
    fail <- function(msg) {
        if (on_error == "stop") {
            stop(msg, call. = FALSE)
        } else {
            warning(msg, call. = FALSE)
            return(invisible(NULL))
        }
    }

    # Check if file_name ends with .rds or .csv and handle accordingly
    if (nchar(file_name) > 4) {
        last_4_chars <- substr(file_name, nchar(file_name) - 3, nchar(file_name))
        if (last_4_chars %in% c(".rds", ".csv")) {
            file_name <- substr(file_name, 1, nchar(file_name) - 4)
            message(
                "File extension (.rds or .csv) not required - adding it is optional."
            )
        }
    }

    # Load data dict
    base_url <- "https://github.com/awiedem/german_election_data/raw/refs/heads/main/data/"
    make_csv <- function(path) paste0(base_url, path, ".csv?download=")
    make_rds <- function(path) paste0(base_url, path, ".rds")

    entries <- list(
        # Original 14 datasets
        list("municipal_unharm",
             "Local elections at the municipal level (1990-2020, unharmonized).",
             "municipal_elections/final/municipal_unharm"),
        list("municipal_harm",
             "Local elections at the municipal level (1990-2020, harmonized).",
             "municipal_elections/final/municipal_harm"),
        list("state_unharm",
             "State elections at the municipal level (2006-2019, unharmonized).",
             "state_elections/final/state_unharm"),
        list("state_harm",
             "State elections at the municipal level (2006-2019, harmonized).",
             "state_elections/final/state_harm"),
        list("federal_muni_raw",
             "Federal elections at the municipal level (1980-2025, raw data).",
             "federal_elections/municipality_level/final/federal_muni_raw"),
        list("federal_muni_unharm",
             "Federal elections at the municipal level (1980-2025, unharmonized).",
             "federal_elections/municipality_level/final/federal_muni_unharm"),
        list("federal_muni_harm_21",
             "Federal elections at the municipal level (1990-2025, harmonized to 2021 boundaries).",
             "federal_elections/municipality_level/final/federal_muni_harm_21"),
        list("federal_muni_harm_25",
             "Federal elections at the municipal level (1990-2025, harmonized to 2025 boundaries).",
             "federal_elections/municipality_level/final/federal_muni_harm_25"),
        list("federal_cty_unharm",
             "Federal elections at the county level (1953-2021, unharmonized).",
             "federal_elections/county_level/final/federal_cty_unharm"),
        list("federal_cty_harm",
             "Federal elections at the county level (1990-2021, harmonized).",
             "federal_elections/county_level/final/federal_cty_harm"),
        list("ags_crosswalks",
             "Crosswalks for municipalities (1990-2025).",
             "crosswalks/final/ags_crosswalks"),
        list("cty_crosswalks",
             "Crosswalks for counties (1990-2025).",
             "crosswalks/final/cty_crosswalks"),
        list("ags_area_pop_emp",
             "Crosswalk covariates (area, population, employment) for municipalities (1990-2025).",
             "covars_municipality/final/ags_area_pop_emp"),
        list("cty_area_pop_emp",
             "Crosswalk covariates (area, population, employment) for counties (1990-2025).",
             "covars_county/final/cty_area_pop_emp"),

        # County (Kreistag) elections
        list("county_elec_unharm",
             "County (Kreistag) elections at the municipal level, unharmonized.",
             "county_elections/final/county_elec_unharm"),
        list("county_elec_harm_21",
             "County (Kreistag) elections, harmonized to 2021 boundaries.",
             "county_elections/final/county_elec_harm_21"),
        list("county_elec_harm_21_cty",
             "County (Kreistag) elections aggregated to county level, harmonized to 2021 boundaries.",
             "county_elections/final/county_elec_harm_21_cty"),
        list("county_elec_harm_21_muni",
             "County (Kreistag) elections at the municipal level, harmonized to 2021 boundaries.",
             "county_elections/final/county_elec_harm_21_muni"),

        # European Parliament elections
        list("european_muni_unharm",
             "European Parliament elections at the municipal level, unharmonized.",
             "european_elections/final/european_muni_unharm"),
        list("european_muni_harm",
             "European Parliament elections at the municipal level, harmonized.",
             "european_elections/final/european_muni_harm"),

        # Mayoral elections
        list("mayoral_unharm",
             "Mayoral election results at the municipal level, unharmonized.",
             "mayoral_elections/final/mayoral_unharm"),
        list("mayoral_harm",
             "Mayoral election results at the municipal level, harmonized.",
             "mayoral_elections/final/mayoral_harm"),
        list("mayoral_candidates",
             "Mayoral candidates (person-level).",
             "mayoral_elections/final/mayoral_candidates"),
        list("mayor_panel",
             "Mayor panel (person-level, one row per mayor-term).",
             "mayoral_elections/final/mayor_panel"),
        list("mayor_panel_harm",
             "Mayor panel (person-level, harmonized to current boundaries).",
             "mayoral_elections/final/mayor_panel_harm"),
        list("mayor_panel_annual",
             "Mayor panel at annual frequency (one row per municipality-year).",
             "mayoral_elections/final/mayor_panel_annual"),
        list("mayor_panel_annual_harm",
             "Mayor panel at annual frequency, harmonized to current boundaries.",
             "mayoral_elections/final/mayor_panel_annual_harm"),

        # Boundary-specific harmonizations
        list("municipal_harm_25",
             "Local elections at the municipal level, harmonized to 2025 boundaries.",
             "municipal_elections/final/municipal_harm_25"),
        list("state_harm_21",
             "State elections at the municipal level, harmonized to 2021 boundaries.",
             "state_elections/final/state_harm_21"),
        list("state_harm_23",
             "State elections at the municipal level, harmonized to 2023 boundaries.",
             "state_elections/final/state_harm_23"),
        list("state_harm_25",
             "State elections at the municipal level, harmonized to 2025 boundaries.",
             "state_elections/final/state_harm_25"),

        # Additional crosswalks
        list("ags_1990_to_2023_crosswalk",
             "Municipality crosswalk: 1990 boundaries to 2023 boundaries.",
             "crosswalks/final/ags_1990_to_2023_crosswalk"),
        list("ags_1990_to_2025_crosswalk",
             "Municipality crosswalk: 1990 boundaries to 2025 boundaries.",
             "crosswalks/final/ags_1990_to_2025_crosswalk"),
        list("crosswalk_ags_2021_to_2023",
             "Municipality crosswalk: AGS 2021 to AGS 2023 (targeted).",
             "crosswalks/final/crosswalk_ags_2021_to_2023"),
        list("crosswalk_ags_2021_2022_to_2023",
             "Municipality crosswalk: AGS 2021 and 2022 to AGS 2023 (targeted).",
             "crosswalks/final/crosswalk_ags_2021_2022_to_2023"),
        list("crosswalk_ags_2023_to_2025",
             "Municipality crosswalk: AGS 2023 to AGS 2025 (targeted; RDS only).",
             "crosswalks/final/crosswalk_ags_2023_to_2025"),
        list("crosswalk_ags_2023_24_to_2025",
             "Municipality crosswalk: AGS 2023 and 2024 to AGS 2025 (targeted; RDS only).",
             "crosswalks/final/crosswalk_ags_2023_24_to_2025"),
        list("crosswalk_ags_2024_to_2025",
             "Municipality crosswalk: AGS 2024 to AGS 2025 (targeted; RDS only).",
             "crosswalks/final/crosswalk_ags_2024_to_2025"),

        # Alternative-boundary covariates
        list("ags_area_pop_emp_2023",
             "Crosswalk covariates (area, population, employment) for municipalities, harmonized to 2023 boundaries.",
             "covars_municipality/final/ags_area_pop_emp_2023")
    )

    data_dictionary <- data.frame(
        data_name   = vapply(entries, `[[`, character(1), 1),
        description = vapply(entries, `[[`, character(1), 2),
        csv_url     = vapply(entries, function(e) make_csv(e[[3]]), character(1)),
        rds_url     = vapply(entries, function(e) make_rds(e[[3]]), character(1)),
        stringsAsFactors = FALSE
    )


    # Check if file_name is in data_dict

    if (!file_name %in% data_dictionary$data_name) {
        # Special handling for deprecated federal_muni_harm dataset
        if (file_name == "federal_muni_harm") {
            return(fail(paste0(
                "The dataset 'federal_muni_harm' has been replaced with two boundary-specific versions:\n",
                "  - 'federal_muni_harm_21': harmonized to 2021 boundaries\n",
                "  - 'federal_muni_harm_25': harmonized to 2025 boundaries\n",
                "Please replace 'federal_muni_harm' in your function call with one of these datasets,\n",
                "depending on which boundary harmonization you need.\n",
                "For a complete list of available datasets, see gerda_data_list()."
            )))
        }

        # Check if there is a close match in data_dict$data_name
        close_matches <- agrep(file_name,
            data_dictionary$data_name,
            max.distance = 0.5, value = TRUE
        )

        if (length(close_matches) > 0) {
            # Prioritize matches that start with the same prefix
            starts_with_prefix <- startsWith(close_matches, file_name)
            if (any(starts_with_prefix)) {
                close_matches <- c(
                    close_matches[starts_with_prefix],
                    close_matches[!starts_with_prefix]
                )
            } else {
                # If no prefix match, use Levenshtein distance
                distances <- stringdist::stringdist(file_name,
                    close_matches,
                    method = "lv"
                )
                close_matches <- close_matches[order(distances)]
            }
        }

        if (length(close_matches) > 1) {
            return(fail(paste0(
                "File name not found in data dictionary.\nDid you mean: \"",
                close_matches[1], "\" or \"", close_matches[2], "\"?\n",
                "For a complete list of available datasets, see gerda_data_list()."
            )))
        } else if (length(close_matches) == 1) {
            return(fail(paste0(
                "File name not found in data dictionary.\nDid you mean: \"",
                close_matches[1], "\"?\n",
                "For a complete list of available datasets, see gerda_data_list()."
            )))
        } else {
            return(fail(paste0(
                "File name not found in data dictionary.\n",
                "For a complete list of available datasets, see gerda_data_list()."
            )))
        }
    }

    if (verbose) {
        message("File name found in data dictionary")
    }

    # Check if file_format is valid
    if (!file_format %in% c("csv", "rds")) {
        return(fail("Invalid file_format. Must be either 'csv' or 'rds'."))
    }

    # Get the url
    url <- data_dictionary[[paste0(file_format, "_url")]][data_dictionary$data_name == file_name]

    if (verbose) {
        message("URL found: ", url)
        message("Loading data...")
    }

    # Download to a tempfile first, then read locally. Streaming `readr::read_rds`
    # directly from a URL breaks on xz-compressed RDS files (e.g. the 1990->2025
    # crosswalks), because the streaming reader doesn't auto-detect xz. Reading
    # from disk lets base `readRDS` auto-detect any R-supported compression, and
    # keeps CSV behavior symmetrical.
    #
    # Raise the download timeout for the duration of the fetch. R's default
    # (60s) is too short for some of the larger GERDA files over GitHub-media
    # on slower connections; users otherwise see sporadic timeouts on the first
    # pull of files like mayor_panel_annual_harm or federal_muni_harm_21.
    old_timeout <- getOption("timeout")
    if (is.null(old_timeout) || old_timeout < 300) {
        options(timeout = 300)
        on.exit(options(timeout = old_timeout), add = TRUE)
    }
    tmp <- tempfile(fileext = paste0(".", file_format))
    on.exit(if (file.exists(tmp)) unlink(tmp), add = TRUE)
    data <- tryCatch(
        {
            utils::download.file(url, tmp, mode = "wb", quiet = !verbose)
            switch(file_format,
                "csv" = read_csv(tmp, show_col_types = FALSE),
                "rds" = readRDS(tmp)
            )
        },
        error = function(e) {
            fail(paste0("Error loading data: ", e$message,
                        "\nThe data may not be available or may have changed. Please contact the package maintainer."))
        }
    )

    # Normalize schema for known upstream inconsistencies.
    # federal_cty_unharm ships with 'ags' (a 5-digit county code) and 'year',
    # while all other county-level datasets use 'county_code' and 'election_year'.
    # To keep downstream helpers like add_gerda_covariates() working without
    # breaking existing user code, add 'county_code'/'election_year' as aliases
    # alongside the original columns. The 'ags' and 'year' aliases are
    # deprecated and scheduled for removal in v0.7.
    if (!is.null(data) && file_name == "federal_cty_unharm") {
        added_alias <- FALSE
        if ("ags" %in% names(data) && !"county_code" %in% names(data)) {
            data$county_code <- data$ags
            added_alias <- TRUE
        }
        if ("year" %in% names(data) && !"election_year" %in% names(data)) {
            data$election_year <- data$year
            added_alias <- TRUE
        }
        if (added_alias) {
            message(
                "Note: 'federal_cty_unharm' now also provides 'county_code' and 'election_year' ",
                "to match other county-level datasets. The upstream 'ags' and 'year' columns ",
                "remain for backwards compatibility but are deprecated and will be removed in v0.7. ",
                "Please migrate your code to the new column names."
            )
        }
    }

    if (verbose) {
        if (!is.null(data)) {
            message("Data loaded successfully")
        } else {
            message("Data loading failed. See warnings for details.")
        }
    }

    return(data)
}

test_that("load_gerda_web fuzzy matching suggests close names", {
    expect_warning(
        result <- load_gerda_web("municipal_harn"), # missing 'm' from 'harm'
        "Did you mean"
    )
    expect_null(result)
    expect_warning(load_gerda_web("municipal_harn"), "gerda_data_list")

    expect_warning(
        result <- load_gerda_web("federal_muni"), # partial match
        "Did you mean"
    )
    expect_null(result)
    expect_warning(load_gerda_web("federal_muni"), "gerda_data_list")

    expect_warning(
        result <- load_gerda_web("state_unhar"), # close to 'state_unharm'
        "Did you mean"
    )
    expect_null(result)
    expect_warning(load_gerda_web("state_unhar"), "gerda_data_list")
})

test_that("deprecated federal_muni_harm returns specific guidance", {
    expect_warning(
        result <- load_gerda_web("federal_muni_harm"),
        "has been replaced with two boundary-specific versions"
    )
    expect_null(result)
    expect_warning(load_gerda_web("federal_muni_harm"), "federal_muni_harm_21")
    expect_warning(load_gerda_web("federal_muni_harm"), "federal_muni_harm_25")
    expect_warning(load_gerda_web("federal_muni_harm"), "Please replace")
    expect_warning(load_gerda_web("federal_muni_harm"), "gerda_data_list")
})

test_that("load_gerda_web handles invalid file names correctly", {
    expect_warning(result <- load_gerda_web("invalid_dataset_name"))
    expect_null(result)

    expect_warning(
        load_gerda_web("nonexistent"),
        "File name not found in data dictionary"
    )
    expect_warning(
        load_gerda_web("nonexistent"),
        "gerda_data_list"
    )
})

test_that("load_gerda_web validates file_format parameter", {
    expect_warning(
        result <- load_gerda_web("municipal_harm", file_format = "xlsx"),
        "Invalid file_format. Must be either 'csv' or 'rds'"
    )
    expect_null(result)

    expect_warning(
        result <- load_gerda_web("municipal_harm", file_format = "json"),
        "Invalid file_format"
    )
    expect_null(result)
})

test_that("load_gerda_web parameter validation", {
    expect_error(
        load_gerda_web("", file_format = "rds")
    )

    expect_error(
        load_gerda_web(NULL, file_format = "rds")
    )

    # Default file_format — skip on CRAN because this triggers a network fetch.
    skip_on_cran()
    expect_silent({
        suppressWarnings(load_gerda_web("municipal_harm"))
    })
})

test_that("load_gerda_web verbose parameter works", {
    skip_on_cran()
    expect_silent({
        suppressWarnings(load_gerda_web("municipal_harm", verbose = FALSE))
    })
})

test_that("load_gerda_web handles both csv and rds formats", {
    skip_on_cran()
    expect_silent({
        suppressWarnings(suppressMessages(
            load_gerda_web("municipal_harm", file_format = "csv", verbose = FALSE)
        ))
    })

    expect_silent({
        suppressWarnings(suppressMessages(
            load_gerda_web("municipal_harm", file_format = "rds", verbose = FALSE)
        ))
    })
})

test_that("on_error = 'stop' converts warnings into errors", {
    # Unknown dataset name: default "warn" yields NULL + warning.
    expect_warning(
        res <- load_gerda_web("this_dataset_does_not_exist"),
        "File name not found"
    )
    expect_null(res)

    # With on_error = "stop", the same input throws.
    expect_error(
        load_gerda_web("this_dataset_does_not_exist", on_error = "stop"),
        "File name not found"
    )

    # Invalid file_format: same promotion behaviour.
    expect_warning(
        res <- load_gerda_web("municipal_harm", file_format = "xlsx"),
        "Invalid file_format"
    )
    expect_null(res)
    expect_error(
        load_gerda_web("municipal_harm", file_format = "xlsx", on_error = "stop"),
        "Invalid file_format"
    )

    # Deprecated federal_muni_harm: warn path returns NULL; stop path errors.
    expect_warning(
        res <- load_gerda_web("federal_muni_harm"),
        "replaced with two boundary-specific versions"
    )
    expect_null(res)
    expect_error(
        load_gerda_web("federal_muni_harm", on_error = "stop"),
        "replaced with two boundary-specific versions"
    )
})

test_that("on_error argument is validated", {
    expect_error(
        load_gerda_web("municipal_harm", on_error = "explode"),
        "on_error must be either"
    )
    expect_error(
        load_gerda_web("municipal_harm", on_error = c("warn", "stop")),
        "on_error must be either"
    )
})

test_that("options(gerda.on_error = 'stop') flips the default", {
    old <- getOption("gerda.on_error")
    on.exit(options(gerda.on_error = old), add = TRUE)
    options(gerda.on_error = "stop")
    expect_error(
        load_gerda_web("this_dataset_does_not_exist"),
        "File name not found"
    )
})

test_that("load_gerda_web detects .rds / .csv extensions", {
    # Triggers a real fetch after stripping the extension; skip on CRAN.
    skip_on_cran()
    expect_message(
        suppressWarnings(result <- load_gerda_web("municipal_harm.rds")),
        "File extension \\(.rds or .csv\\) not required - adding it is optional"
    )

    expect_message(
        suppressWarnings(result <- load_gerda_web("municipal_harm.csv")),
        "File extension \\(.rds or .csv\\) not required - adding it is optional"
    )

    # Extension does not need to match file_format arg
    expect_message(
        suppressWarnings(result <- load_gerda_web("municipal_harm.rds", file_format = "csv")),
        "File extension \\(.rds or .csv\\) not required - adding it is optional"
    )

    expect_message(
        suppressWarnings(result <- load_gerda_web("municipal_harm.csv", file_format = "rds")),
        "File extension \\(.rds or .csv\\) not required - adding it is optional"
    )
})

test_that("load_gerda_web treats extensions as equivalent to bare name", {
    skip_on_cran()
    test_cases <- c(
        "municipal_harm",
        "municipal_harm.rds",
        "municipal_harm.csv"
    )

    for (test_case in test_cases) {
        expect_silent({
            suppressWarnings(suppressMessages(load_gerda_web(test_case, verbose = FALSE)))
        })
    }
})

test_that("load_gerda_web rejects names that merely contain '.rds'/'.csv'", {
    expect_warning(
        suppressMessages(result <- load_gerda_web("some_dataset_with_rds_in_name")),
        "File name not found in data dictionary"
    )
    expect_null(result)

    # Extension detection looks only at the last 4 characters
    expect_warning(
        suppressMessages(result <- load_gerda_web("municipal_harm.rds.backup")),
        "File name not found in data dictionary"
    )
    expect_null(result)
})

test_that("load_gerda_web extension message content is informative", {
    skip_on_cran()
    expect_message(
        suppressWarnings(load_gerda_web("municipal_harm.rds", file_format = "rds")),
        "File extension \\(.rds or .csv\\) not required - adding it is optional"
    )

    expect_message(
        suppressWarnings(load_gerda_web("municipal_harm.csv", file_format = "csv")),
        "File extension \\(.rds or .csv\\) not required - adding it is optional"
    )
})

---
title: "Introduction to gerda"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to gerda}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(gerda)
```

## Overview

The `gerda` package provides functions to access and work with GERDA datasets. The German Election Database (GERDA) provides data on German elections spanning federal elections (since 1953 at the county level, 1980 at the municipal level), state (*Landtag*) elections, local (*Kommunal*) elections, mayoral (*Bürgermeister*) elections, European Parliament elections, and county (*Kreistag*) elections. All election datasets include turnout and vote shares for all major parties. GERDA also supplies geographically harmonized datasets that account for changes in municipal boundaries and mail-in voting districts.

In addition to election results, the package provides county-level socioeconomic covariates from INKAR, municipality-level data from the German Census 2022, and a party crosswalk that maps GERDA party names to standardized ParlGov attributes.

GERDA was compiled by Vincent Heddesheimer, Florian Sichart, Andreas Wiedemann and Hanno Hilbig. For additional information, see also the GERDA website (www.german-elections.com) and the accompanying publication: [doi.org/10.1038/s41597-025-04811-5](https://doi.org/10.1038/s41597-025-04811-5)

This vignette will introduce you to the main functions of the package and demonstrate how to use them. 

## Available Datasets

To see a list of all available GERDA electoral result datasets, you can use the `gerda_data_list()` function:

```{r}
gerda_data_list()
```

This function displays a formatted table with the names and descriptions of all available datasets. You can use the `file_name` column from this output to specify which dataset you want to load using the `load_gerda_web()` function.

## Loading Data

The main function for loading GERDA data is `load_gerda_web()`. This function allows you to load a specific dataset from a web source. Here's an example of how to use it:

```{r, eval=FALSE}
# Load the municipal harmonized dataset
municipal_harm_data <- load_gerda_web("municipal_harm", verbose = TRUE, file_format = "rds")
```

The `load_gerda_web()` function takes the following parameters:

- `file_name`: A character string with the name of the dataset to load, e.g. `"federal_cty_harm"` (as shown in the `gerda_data_list()` output). The function supports fuzzy matching, so close misspellings will produce a helpful suggestion.
- `verbose`: If set to `TRUE`, it prints messages about the loading process (default is `FALSE`)
- `file_format`: Specifies the format of the file to load, either `"rds"` or `"csv"` (default is `"rds"`). Both formats return the same tibble, so this choice only affects download size and speed.

## Example Workflow

Here's an example of a typical workflow using the `gerda` package:

1. List available datasets:

```{r}
gerda_data_list()
```

2. Load a dataset (in this case, the federal elections at the county level, harmonized):

```{r, eval=FALSE}
federal_cty_harm <- load_gerda_web("federal_cty_harm", verbose = TRUE)
```

## Joining GERDA Datasets

If you are using `add_gerda_covariates()` or `add_gerda_census()`, you can skip this section: the helpers detect the level of your data and use the correct join keys automatically. If you are writing a manual `left_join()` or merging against other sources, the table below shows which identifier and time columns each family carries.

| Dataset family                                                                                        | Geographic id                                      | Time column                  |
|------------------------------------------------------------------------------------------------------|----------------------------------------------------|------------------------------|
| `municipal_*`, `state_*`, `federal_muni_*`, `european_muni_*`, `mayoral_*`                            | `ags` (8-digit municipality)                       | `election_year` (+ `election_date` where available) |
| `federal_cty_harm`                                                                                   | `county_code` (5-digit county)                     | `election_year`              |
| `federal_cty_unharm`                                                                                 | `county_code` + `ags` alias (see Deprecations)     | `election_year` + `year` alias |
| `county_elec_*` (with `_cty` suffix)                                                                  | `county_code` (5-digit county)                     | `election_year`              |
| `county_elec_*` (without `_cty` suffix)                                                               | `ags` (8-digit municipality)                       | `election_year`              |
| `mayor_panel` / `mayor_panel_harm`                                                                    | `ags` + `person_id`                                | `election_date`              |
| `mayor_panel_annual` / `mayor_panel_annual_harm`                                                     | `ags` + `person_id`                                | `year`                       |
| `gerda_covariates()` (INKAR, county-level)                                                            | `county_code` (5-digit county)                     | `year` (not `election_year`) |
| `gerda_census()` (Zensus 2022, municipality-level)                                                    | `ags` (8-digit municipality)                       | *time-invariant (2022)*      |
| `ags_crosswalks`, `ags_1990_to_2023_crosswalk`, `ags_1990_to_2025_crosswalk`, `crosswalk_ags_*`       | Pair of AGS codes at source and target vintages    | Vintage is encoded in column names |
| `cty_crosswalks`                                                                                      | Pair of 5-digit county codes at source and target  | Vintage is in column names   |

Two things to watch for when joining manually:

- `gerda_covariates()` uses `year`, not `election_year`. If you merge it directly into federal election data, you need a rename: `by = c("county_code" = "county_code", "election_year" = "year")`.
- On municipal-level files, the `county` column is a county name or partial code, not the 5-digit AGS-based county code. Use `substr(ags, 1, 5)` to extract the county key when you want to join against county-level data.

## County-Level Covariates

The `gerda` package includes county-level socioeconomic and demographic covariates from INKAR (Indikatoren und Karten zur Raum- und Stadtentwicklung). These covariates can be easily merged with GERDA election data to enrich your analyses. INKAR data is available from 1995 to 2022, so covariates can be matched to federal elections from 1998 onwards (earlier elections fall outside the INKAR coverage window).

### Quick Start

The easiest way to add covariates to your election data is using the `add_gerda_covariates()` function:

```{r, eval=FALSE}
library(dplyr)

# Load election data and add covariates
merged <- load_gerda_web("federal_cty_harm") %>%
  add_gerda_covariates()

# Your data now includes 30 county-level covariates!
```

Under the hood, `add_gerda_covariates()` merges on county code and election year. It automatically:

- Detects whether the input is county-level or municipal-level data (and extracts county codes from municipal AGS codes if needed)
- Performs a left join, so all election rows are kept and covariates are added where available
- Validates that the required columns (`county_code` or `ags`, and `election_year`) are present

### Available Covariates

The covariates dataset includes 30 variables across 10 categories (for the full list of variable names, units, and descriptions, see `gerda_covariates_codebook()`):

- **Demographics**: Age structure, foreign population, gender composition
- **Economy**: GDP per capita, sectoral composition, enterprise structure
- **Labor Market**: Unemployment rates (overall, youth, long-term)
- **Education**: School completion rates, students, apprentices
- **Income**: Purchasing power, low-income households
- **Healthcare**: Physician density, hospital beds, GP density
- **Childcare**: Coverage rates for under-3 and 3-6 age groups
- **Housing**: Building permits, rent levels, living space
- **Transport**: Cars per capita
- **Public Finances**: Municipal debt, tax revenue

### Viewing the Codebook

To see detailed information about each covariate, including units and missing data patterns:

```{r, eval=FALSE}
# Get the codebook
codebook <- gerda_covariates_codebook()
print(codebook)

# Find variables with good coverage
library(dplyr)
codebook %>%
  filter(missing_pct < 10) %>%
  select(variable, label, category)
```

### Advanced Usage

For more control, you can access the raw covariates data:

```{r, eval=FALSE}
# Get raw covariate data
covs <- gerda_covariates()

# Inspect before merging
summary(covs$unemployment_rate)

# Custom merge
elections <- load_gerda_web("federal_cty_harm")
merged <- elections %>%
  left_join(covs, by = c("county_code" = "county_code", "election_year" = "year"))
```

### Data Coverage

- **Counties**: 400 German counties (Kreise)
- **Time period**: 1995-2022 (annual data)
- **Election coverage**: Elections from 1998 onwards have covariate data

Coverage varies by variable: core indicators (demographics, economy, labor market) are available for all 7 federal election years (1998-2021). Newer INKAR indicators (e.g., childcare, some healthcare variables) are available for 2-3 recent elections only. Consult the codebook's `missing_pct` column to check per-variable availability before analysis.

## Census 2022 Data

The `gerda` package includes municipality-level data from the German Census 2022 (Zensus 2022). This cross-sectional snapshot covers approximately 10,800 municipalities and can be merged with any GERDA election dataset.

The main advantage of this covariate data is that it is observed at the municipal level (unlike the county-level INKAR data). This allows for more fine-grained analyses of local election outcomes. However, the census is a single time point (2022), so it does not vary across election years. This means that the resulting merged dataset will have time-invariant covariates, i.e. each municipality receives the same census values for all election years. Users should not conduct analyses that rely on over-time variation in these covariates.

### Quick Start

```{r, eval=FALSE}
library(gerda)

# Add census data to municipal-level elections
muni_merged <- load_gerda_web("federal_muni_harm_21") |>
  add_gerda_census()

# Also works with county-level data (aggregated from municipalities)
county_merged <- load_gerda_web("federal_cty_harm") |>
  add_gerda_census()
```

### Available Indicators

The census data includes 14 indicators across four categories:

- **Demographics**: Population, age structure (under 18, 18-29, 30-49, 50-64, 65+)
- **Migration**: Migration background share, foreign nationals share
- **Households**: Average household size
- **Housing**: Total dwellings, vacancy rate, ownership rate, average rent per m², single-family home share

Since the census is a 2022 snapshot, the same values are attached to all election years (see also the note above). 

### Viewing the Codebook

```{r, eval=FALSE}
# Get the census codebook
census_cb <- gerda_census_codebook()
print(census_cb)
```

### Data Coverage

Most census variables have >95% municipality coverage. `avg_household_size_census22` has approximately 12.5% missing values because Destatis suppresses data for small municipalities under its disclosure rules.

## Party Crosswalk Function

The `party_crosswalk()` function provides a mapping between GERDA party names and standardized party information from the ParlGov database. This is particularly useful for linking GERDA data with other political science datasets or for obtaining standardized party characteristics.

### Usage

The function takes two main parameters:

- `party_gerda`: A character vector of GERDA party names
- `destination`: The name of the column from the ParlGov view_party table to map to

### Available Mapping Options

You can map GERDA party names to various standardized party characteristics, including:

- `left_right`: Left-right position scores
- `party_name_english`: English party names
- `party_name_short`: Short party names
- `country_name`: Country names
- And many other ParlGov variables

### Example

```{r, eval=FALSE}
# Map GERDA party names to left-right positions
parties <- c("cdu", "spd", "linke_pds", "fdp")
left_right_scores <- party_crosswalk(parties, "left_right")
print(left_right_scores)

# Map to English party names
english_names <- party_crosswalk(parties, "party_name_english")
print(english_names)
```

This function is especially useful when you want to:

- Analyze parties along ideological dimensions
- Merge GERDA data with other comparative datasets
- Standardize party names across different data sources
- Access additional party metadata from ParlGov

## Conclusion

The `gerda` package provides easy access to a wide range of German election and related data. By using the `gerda_data_list()` function to explore available datasets and `load_gerda_web()` to load them, you can quickly incorporate this data into your research or analysis projects.

For more information or to provide feedback, please contact <hhilbig@ucdavis.edu> or visit the GitHub repository at https://github.com/hhilbig/gerda.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/party_crosswalk.R
\name{party_crosswalk}
\alias{party_crosswalk}
\title{Map GERDA Party Names to ParlGov Attributes}
\usage{
party_crosswalk(party_gerda, destination)
}
\arguments{
\item{party_gerda}{A character vector containing the GERDA party names to be converted.}

\item{destination}{A single string naming the target column. Available destinations:
\itemize{
\item \strong{Names}: \code{party_name}, \code{party_name_ascii}, \code{party_name_short}, \code{party_name_english}
\item \strong{Party family}: \code{family_name}, \code{family_name_short}
\item \strong{Ideology scales (ParlGov)}: \code{left_right}, \code{state_market}, \code{liberty_authority}, \code{eu_anti_pro}
\item \strong{External ideology scores}: \code{cmp}, \code{euprofiler}, \code{ees}, \code{castles_mair}, \code{huber_inglehart}, \code{ray}, \code{benoit_laver}, \code{chess}
\item \strong{Identifiers}: \code{country_id}, \code{party_id}, \code{family_id}
}}
}
\value{
A vector of the same length as \code{party_gerda} with the mapped values.
}
\description{
Creates a crosswalk between GERDA party names and ParlGov's \code{view_party} attributes.
If a party name is not found, the corresponding output element is \code{NA}. This function
expects GERDA party names (lowercase, underscores); other naming schemes will mostly
return \code{NA}.
}
\examples{
party_crosswalk(c("cdu", "spd", "linke_pds", NA), "left_right")
party_crosswalk(c("cdu", "afd"), "family_name_short")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gerda_census.R
\name{gerda_census_codebook}
\alias{gerda_census_codebook}
\title{Get Codebook for Census 2022 Data}
\usage{
gerda_census_codebook()
}
\value{
A data frame with 16 rows documenting all variables in the census
dataset.
}
\description{
Returns the data dictionary for municipality-level Census 2022 indicators.
Provides variable names, labels, units, and data sources.
}
\examples{
# View the codebook
codebook <- gerda_census_codebook()
print(codebook)

}
\seealso{
\code{\link{gerda_census}} for the actual census data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gerda_data_list.R
\name{gerda_data_list}
\alias{gerda_data_list}
\title{List of GERDA Data}
\usage{
gerda_data_list(print_table = TRUE)
}
\arguments{
\item{print_table}{A logical value indicating whether to print the table in the console (TRUE) or return the data as a tibble (FALSE). Default is TRUE.}
}
\value{
A tibble containing the available GERDA data with descriptions. When print_table = TRUE,
the function prints a formatted table to the console and invisibly returns the data tibble.
When print_table = FALSE, the function directly returns the data tibble.
}
\description{
This function lists the available GERDA data sets. The purpose of this
function is to quickly provide a list of available data sets and their
descriptions.
}
\details{
In addition to downloadable datasets, the package includes bundled
covariate data accessible via dedicated functions:
\itemize{
\item \code{\link{gerda_covariates}}: County-level INKAR covariates (1995-2022)
\item \code{\link{gerda_census}}: Municipality-level Census 2022 data
}
}
\examples{
gerda_data_list()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gerda_covariates.R
\name{add_gerda_covariates}
\alias{add_gerda_covariates}
\title{Add County-Level Covariates to GERDA Election Data}
\usage{
add_gerda_covariates(election_data)
}
\arguments{
\item{election_data}{A data frame containing GERDA election data. Must
contain a column with county or municipal codes (see Details) and
\code{election_year}.}
}
\value{
The input data frame with additional columns for all 30 county-level
covariates. The number of rows remains unchanged (left join).
}
\description{
Convenience function to merge INKAR county-level (Kreis) covariates with
GERDA election data. This is the recommended way to add covariates, as it
automatically uses the correct join keys and prevents common merge errors.

The function works with both county-level and municipal-level election data:
\itemize{
\item \strong{County-level data}: Direct merge using county codes
\item \strong{Municipal-level data}: Automatically extracts county code
from municipal AGS (first 5 digits) and merges
}

\strong{Important}: Covariates are always at the county level. When merging
with municipal data, all municipalities within the same county will receive
identical covariate values.

The function performs a left join, keeping all rows from the election data
and adding covariates where available. This automatically retains only
election years.
}
\details{
\subsection{Required Columns}{

The input data must contain \code{election_year} and one of:
\itemize{
\item \code{county_code}: 5-digit county code (AGS) for county-level data
\item \code{ags}: 8-digit municipal code (AGS) for municipal-level data
}

The function automatically detects which column is present and performs the
appropriate merge. For municipal data, the county code is extracted from the
first 5 digits of the AGS.
}

\subsection{Data Level}{

Covariates are at the county (Kreis) level:
\itemize{
\item \strong{County-level merge}: One-to-one match, each county gets its covariates
\item \strong{Municipal-level merge}: Many-to-one match, all municipalities in the
same county receive identical covariate values
}
}

\subsection{Data Availability}{

Covariates are available from 1995-2022. For GERDA federal elections:
\itemize{
\item Elections 1990, 1994: No covariates (before 1995)
\item Elections 1998-2021: Covariates available
}
}

\subsection{Missing Data}{

Some covariates have missing values. Use \code{gerda_covariates_codebook()}
to check data availability for specific variables.
}
}
\examples{
\dontrun{
library(gerda)
library(dplyr)

# Example 1: County-level election data
county_data <- load_gerda_web("federal_cty_harm") \%>\%
  add_gerda_covariates()

# Check the result
names(county_data) # See new covariate columns
table(county_data$election_year) # Only election years

# Example 2: Municipal-level election data
# Note: All municipalities in the same county will get identical covariates
muni_data <- load_gerda_web("federal_muni_harm_21") \%>\%
  add_gerda_covariates()

# Verify: municipalities in same county have same covariate values.
# The county code is the first 5 digits of the 8-digit municipal AGS.
muni_data \%>\%
  mutate(county_code = substr(ags, 1, 5)) \%>\%
  group_by(county_code, election_year) \%>\%
  summarize(
    n_munis = n(),
    unemp_range = max(unemployment_rate) - min(unemployment_rate)
  )

# Analyze with covariates
county_data \%>\%
  filter(election_year == 2021) \%>\%
  filter(!is.na(unemployment_rate)) \%>\%
  summarize(cor_unemployment_afd = cor(unemployment_rate, afd))
}

}
\seealso{
\itemize{
\item \code{\link{gerda_covariates}} for direct access to the covariate data
\item \code{\link{gerda_covariates_codebook}} for variable descriptions
\item \code{\link{load_gerda_web}} for loading GERDA election data
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_gerda_web.R
\name{load_gerda_web}
\alias{load_gerda_web}
\title{Load GERDA Data}
\usage{
load_gerda_web(
  file_name,
  verbose = FALSE,
  file_format = "rds",
  on_error = getOption("gerda.on_error", "warn")
)
}
\arguments{
\item{file_name}{A character string specifying the name of the file to load. For a list of available data, see \code{\link{gerda_data_list}}.}

\item{verbose}{A logical value indicating whether to print additional messages to the console. Default is FALSE.}

\item{file_format}{A character string specifying the format of the file. Must be either "csv" or "rds". Default is "rds".}

\item{on_error}{How to handle errors (unknown dataset name, failed download, corrupt file, invalid \code{file_format}). Either \code{"warn"} (default) to emit a warning and return \code{NULL}, or \code{"stop"} to raise an error. Use \code{"stop"} inside scripts or pipelines where silent \code{NULL} returns would produce confusing downstream failures. The global default can also be overridden with \code{options(gerda.on_error = "stop")}.}
}
\value{
A tibble containing the loaded data, or NULL if the data could not be loaded.
}
\description{
This function loads GERDA data from a web source.
}
\section{Vote-share columns}{

Election datasets expose one column per party (e.g. \code{cdu}, \code{spd}, \code{gruene}, \code{afd}).
These columns hold the party's share of valid votes and are expressed as
fractions of 1. They do \strong{not} sum to 1 across the named major parties:
the remainder is held by smaller parties with their own columns and, at
the tail, an \code{other} category. For example, in \code{federal_cty_harm} for 2021,
\code{cdu + csu + spd + gruene + fdp + linke_pds + afd} is typically around
0.91 and ranges roughly 0.78 to 0.97 across counties. To reconstruct a
full 1.0 share, include every party column or use \code{other} together with
turnout and invalid-vote columns.
}

\examples{
\donttest{
# Load harmonized municipal elections data
data_municipal_harm <- load_gerda_web("municipal_harm", verbose = TRUE, file_format = "rds")

# Load federal election data harmonized to 2025 boundaries (includes 2025 election)
data_federal_2025 <- load_gerda_web("federal_muni_harm_25", verbose = TRUE, file_format = "rds")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gerda_covariates.R
\name{gerda_covariates_codebook}
\alias{gerda_covariates_codebook}
\title{Get Codebook for County-Level Covariates}
\usage{
gerda_covariates_codebook()
}
\value{
A data frame with 32 rows documenting all variables in the county
covariates dataset.
}
\description{
Returns the data dictionary for county-level (Kreis) covariates from INKAR.
Provides variable names, labels, units, categories, original INKAR codes,
and missing data information for all county-level socioeconomic and
demographic indicators.
}
\examples{
# View the full codebook
codebook <- gerda_covariates_codebook()
print(codebook)

# Find variables by category
library(dplyr)
codebook \%>\%
  filter(category == "Demographics")

# Find variables with good coverage
codebook \%>\%
  filter(missing_pct < 5)

}
\seealso{
\code{\link{gerda_covariates}} for the actual covariate data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gerda_census.R
\name{gerda_census}
\alias{gerda_census}
\title{Get Municipality-Level Census 2022 Data}
\usage{
gerda_census()
}
\value{
A data frame with approximately 10,800 rows (one per municipality)
and 16 columns containing census indicators. See
\code{\link{gerda_census_codebook}} for variable descriptions.
}
\description{
Returns municipality-level demographic and socioeconomic data from the
German Census 2022 (Zensus 2022). This is a cross-sectional snapshot
covering all German municipalities.

For most users, we recommend using \code{\link{add_gerda_census}} instead,
which automatically merges census data with GERDA election data.
}
\details{
The dataset includes:
\itemize{
\item Demographics: Population, age structure
\item Migration: Migration background, foreign nationals
\item Households: Average household size
\item Housing: Dwellings, vacancy, ownership, rents, building types
}

Municipality codes are 8-digit AGS codes. Since the census is a single
2022 snapshot, there is no year dimension.
}
\examples{
# Get the census data
census <- gerda_census()
head(census)

# Check available municipalities
nrow(census)

}
\seealso{
\itemize{
\item \code{\link{add_gerda_census}} for automatic merging with election data
\item \code{\link{gerda_census_codebook}} for variable descriptions
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gerda_census.R
\name{add_gerda_census}
\alias{add_gerda_census}
\title{Add Census 2022 Data to GERDA Election Data}
\usage{
add_gerda_census(election_data)
}
\arguments{
\item{election_data}{A data frame containing GERDA election data. Must contain
either an \code{ags} column (municipality level) or a \code{county_code}
column (county level).}
}
\value{
The input data frame with additional census columns appended. The
number of rows remains unchanged (left join).
}
\description{
Convenience function to merge Zensus 2022 municipality-level data with
GERDA election data. The census provides a cross-sectional snapshot (2022),
so the same values are attached to all election years.

The function works with both municipality-level and county-level election data:
\itemize{
\item \strong{Municipality-level data}: Direct merge using 8-digit AGS codes
\item \strong{County-level data}: Census data is aggregated to the county level
(population-weighted means for shares, sums for counts) before merging
}
}
\details{
\subsection{Required Columns}{

The input data must contain one of:
\itemize{
\item \code{ags}: 8-digit municipal code for municipality-level data
\item \code{county_code}: 5-digit county code for county-level data
}
}

\subsection{Merge Behavior}{

Since the census is a 2022 cross-section, census values are the same for all
election years. The merge is on geography only (no year join).

For county-level data, municipality-level census data is first aggregated:
\itemize{
\item Share variables: Population-weighted means
\item Count variables (population_census22, total_dwellings_census22): Sums
\item Other variables (avg_household_size_census22, avg_rent_per_m2_census22): Population-weighted means
}
}
}
\examples{
\dontrun{
library(gerda)

# Municipality-level merge
muni_data <- load_gerda_web("federal_muni_harm_21") |>
  add_gerda_census()

# County-level merge (aggregated from municipalities)
county_data <- load_gerda_web("federal_cty_harm") |>
  add_gerda_census()
}

}
\seealso{
\itemize{
\item \code{\link{gerda_census}} for direct access to the census data
\item \code{\link{gerda_census_codebook}} for variable descriptions
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gerda_covariates.R
\name{gerda_covariates}
\alias{gerda_covariates}
\title{Get County-Level Covariates from INKAR}
\usage{
gerda_covariates()
}
\value{
A data frame with 11,200 rows and 32 columns containing county-level
covariates for 400 German counties from 1995 to 2022. See
\code{\link{gerda_covariates_codebook}} for variable descriptions.
}
\description{
Returns county-level socioeconomic and demographic covariates from INKAR.
This function provides flexible access to the raw covariate data for
advanced users who want to inspect or manipulate it before merging with
county-level election data.

For most users, we recommend using \code{\link{add_gerda_covariates}} instead,
which automatically performs the merge with correct join keys.

\strong{Note}: These covariates are at the county (Kreis) level and should be
merged with county-level GERDA data (e.g., \code{federal_cty_harm}).
}
\details{
The dataset includes 30 socioeconomic and demographic variables:
\itemize{
\item Demographics: Age structure, foreign population, gender
\item Economy: GDP, sectoral composition, enterprise structure
\item Labor Market: Unemployment rates (overall, youth, long-term)
\item Education: School completion rates, students, apprentices
\item Income: Purchasing power, low-income households
\item Healthcare: Physician density, hospital beds, GP density
\item Childcare: Coverage rates for under-3 and 3-6 age groups
\item Housing: Building permits, rent levels, living space
\item Transport: Cars per capita
\item Public Finances: Municipal debt, tax revenue
}

County codes are formatted as 5-digit AGS codes matching GERDA's harmonized
county codes (2021 boundaries).
}
\examples{
# Get the covariates data (bundled, no network call)
covs <- gerda_covariates()

# Inspect the data
head(covs)
summary(covs)

\donttest{
# Manual merge (advanced) — downloads election data from GitHub
library(dplyr)
elections <- load_gerda_web("federal_cty_harm")
merged <- elections \%>\%
  left_join(covs, by = c("county_code" = "county_code", "election_year" = "year"))
}

}
\seealso{
\itemize{
\item \code{\link{add_gerda_covariates}} for automatic merging (recommended)
\item \code{\link{gerda_covariates_codebook}} for variable descriptions
}
}
